# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from djangoplus.ui.components.report import Timeline, StatisticsTable


def timeline(value, **kwargs):
    return Timeline(kwargs.get('request'),  kwargs.get('verbose_name'), value)


def statistics(value, **kwargs):
    return StatisticsTable(kwargs.get('request'), kwargs.get('verbose_name'), value)


def chart(value, **kwargs):
    return StatisticsTable(kwargs.get('request'), kwargs.get('verbose_name'), value).as_chart()


def pie_chart(value, **kwargs):
    return chart(value, **kwargs).pie()


def donut_chart(value, **kwargs):
    return chart(value, **kwargs).donut()


def bar_chart(value, **kwargs):
    return chart(value, **kwargs).bar()


def horizontal_bar_chart(value, **kwargs):
    return chart(value, **kwargs).horizontal_bar()


def stack_chart(value, **kwargs):
    return chart(value, **kwargs).stack()


def horizontal_stack_chart(value, **kwargs):
    return chart(value, **kwargs).horizontal_stack()


def line_chart(value, **kwargs):
    return chart(value, **kwargs).line()


def area_chart(value, **kwargs):
    return chart(value, **kwargs).area()


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from djangoplus import cache
from djangoplus.utils.metadata import set_metadata, iterable


def meta(verbose_name, can_view=(), formatter=None, dashboard=None):
    def decorate(func):
        set_metadata(func, 'type', 'attr')
        set_metadata(func, 'verbose_name', verbose_name)
        set_metadata(func, 'can_view', can_view)
        set_metadata(func, 'formatter', formatter)
        set_metadata(func, 'position', dashboard)
        return func
    return decorate


def subset(title, can_view=(), alert=False, notify=None, menu=None, sequence=0):
    def decorate(function):
        set_metadata(function, 'type', 'subset')
        set_metadata(function, 'tab', True)
        set_metadata(function, 'title', title)
        set_metadata(function, 'alert', alert)
        set_metadata(function, 'notify', notify)
        set_metadata(function, 'menu', menu)
        set_metadata(function, 'sequence', sequence)
        set_metadata(function, 'can_view', iterable(can_view))
        set_metadata(function, 'name', function.func_name)
        set_metadata(function, 'order', cache.next_number())
        return function

    return decorate


def action(title, can_execute=(), condition=None, category='Ações', style='popup', input=None,
           message='Ação realizada com sucesso.', initial=None, choices=None, inline=None, icon=None,
           sequence=0, can_execute_by_organization=None, can_execute_by_unit=None, can_execute_by_role=None, redirect_to=None, menu=None):
    def decorate(function):
        function._action = dict(
            title=title, can_execute=iterable(can_execute),
            input=input, group=category or title, css=style, condition=condition, view_name=function.__name__,
            message=message, initial=initial or '{}_initial'.format(function.__name__), function=function,
            choices=choices or '{}_choices'.format(function.__name__), inline=inline, icon=icon, doc=function.__doc__,
            sequence=sequence, can_execute_by_organization=iterable(can_execute_by_organization),
            can_execute_by_unit=iterable(can_execute_by_unit), can_execute_by_role=iterable(can_execute_by_role),
            redirect_to=redirect_to, menu=menu
        )
        return function

    return decorate


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from decimal import Decimal
from djangoplus.ui import Component
from djangoplus.utils.metadata import get_fiendly_name, get_field, get_metadata, getattr2
from djangoplus.ui.components import forms
from django.conf import settings

PERCENT_SYMBOL = '%'
DOLLAR_SYMBOL = '$'
REAIS_SYMBOL = 'R$'


class Chart(Component):
    def __init__(self, labels, series, groups=[], symbol=None, title=None):

        super(Chart, self).__init__()

        self.type = 'bar'
        self.labels = labels
        self.series = series
        self.groups = groups
        self.symbol = symbol or ''
        self.title = title

        for i, serie in enumerate(self.series):
            for j, valor in enumerate(serie):
                if type(valor) == Decimal:
                    self.series[i][j] = float(valor)

        self.colors = []
        for color in settings.GRADIENT:
            self.colors.append(color)
        for i in range(len(self.colors), 16):
            self.colors.append('#FFFFFF')

    def __unicode__(self):
        return self.render('chart.html')

    def change_type(self, chart_type):
        self.type = chart_type
        return self

    def pie(self):
        return self.change_type('pie')

    def donut(self):
        return self.change_type('donut')

    def bar(self):
        return self.change_type('bar')

    def horizontal_bar(self):
        return self.change_type('horizontalbar')

    def stack(self):
        return self.change_type('stack')

    def horizontal_stack(self):
        return self.change_type('horizontalstack')

    def line(self):
        return self.change_type('line')

    def area(self):
        return self.change_type('area')


class Timeline(Component):

    def __init__(self, request, description, items=[]):
        super(Timeline, self).__init__(request)
        self.items = items
        self.description = description
        self.width = 0

    def add(self, status, date):
        self.items.append((status, date))

    def __unicode__(self):
        self.width = str(100.0/len(self.items))
        return self.render('timeline.html')


class Table(Component):
    def __init__(self, request, title, header=[], rows=[], footer=[], enumerable=True):
        super(Table, self).__init__(request)
        self.title = title
        self.header = header
        self.rows = rows
        self.footer = footer
        self.enumerable = enumerable

    def __unicode__(self):
        return self.render('table.html')


class ModelTable(Table):
    def __init__(self, request, title, qs, list_display=()):

        header = []
        rows = []

        if not list_display:
            list_display = get_metadata(qs.model, 'list_display')

        for lookup in list_display:
            header.append(get_fiendly_name(qs.model, lookup, as_tuple=False))

        for obj in qs:
            row = []
            for lookup in list_display:
                row.append(getattr2(obj, lookup))
            rows.append(row)

        super(ModelTable, self).__init__(request, title, header, rows)


class StatisticsTable(Table):
    def __init__(self, request, title, queryset_statistics, symbol=None):
        if queryset_statistics.groups:
            header = [''] + queryset_statistics.labels + ['']
            rows = []
            for i, serie in enumerate(queryset_statistics.series):
                rows.append([queryset_statistics.groups[i]]+serie+[queryset_statistics.xtotal[i]])
            footer = [''] + queryset_statistics.ytotal + [queryset_statistics.total()]
        else:
            header = []
            rows = []
            footer = []
            if queryset_statistics.series:
                for i, serie in enumerate(queryset_statistics.series[0]):
                    rows.append([queryset_statistics.labels[i], queryset_statistics.series[0][i]])
                if len(queryset_statistics.series[0]) > 1:
                    footer.append('')
                    footer.append(queryset_statistics.total())
        super(StatisticsTable, self).__init__(request, title, header, rows, footer, enumerable=False)
        self.queryset_statistics = queryset_statistics
        self.symbol = symbol

    def as_chart(self):
        if not self.queryset_statistics.groups:
            return Chart(self.queryset_statistics.labels, self.queryset_statistics.series, symbol=self.symbol, title=self.title).donut()
        else:
            chart = Chart(self.queryset_statistics.labels, self.queryset_statistics.series, self.queryset_statistics.groups, symbol=self.symbol, title=self.title)
            if self.queryset_statistics.labels and self.queryset_statistics.labels[0] == 'Jan':
                return chart.line()
            else:
                return chart.bar()


class ModelReport(Component):
    def __init__(self, request, title, qs, list_display=(), list_filter=()):
        super(ModelReport, self).__init__(request)
        self.title = title
        self.qs = qs
        self.components = []
        self.filters = []

        if list_filter:
            form = forms.Form(request, method='GET')
            form.icon = 'fa-file-text-o'
            form.title = ''
            form.submit_label = 'Gerar Relatório'
            for field_name in list_filter:
                field = get_field(qs.model, field_name)
                if hasattr(field, 'choices') and field.choices:
                    form.fields[field_name] = forms.ChoiceField(choices=[['', '']]+field.choices, label=field.verbose_name, required=False)
                else:
                    form.fields[field_name] = forms.ModelChoiceField(field.rel.to.objects.all(), label=field.verbose_name, required=False)
            if form.is_valid():
                for field_name in list_filter:
                    value = form.cleaned_data[field_name]
                    if value:
                        qs = qs.filter(**{field_name : value})
                        self.filters.append((form.fields[field_name].label, value))
            self.form = form

        order_by = get_metadata(qs.model, 'order_by', iterable=True)
        if order_by:
            qs = qs.order_by(*order_by)
        table_description = get_metadata(qs.model, 'verbose_name_plural')
        self.table = ModelTable(request, table_description, qs, list_display)

    def count(self, vertical_key, horizontal_key=None, symbol=None, add_table=True, add_chart=False):
        component = StatisticsTable(self.request, 'Resumo', self.qs.count(vertical_key, horizontal_key), symbol)
        if add_table:
            self.components.append(component)
        if add_chart:
            self.components.append(component.as_chart())

    def __unicode__(self):
        return self.render('model_report.html')








# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# Generated by Django 1.10 on 2016-09-14 18:21
from __future__ import unicode_literals

from django.db import migrations
import django.db.models.deletion
import djangoplus.admin.models
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('admin', '0002_log_logindex'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentType',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('contenttypes.contenttype',),
            managers=[
                ('objects', djangoplus.admin.models.ContentTypeManager()),
            ],
        ),
        migrations.AlterField(
            model_name='log',
            name='content_type',
            field=djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='admin.ContentType', verbose_name='Objeto'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# Generated by Django 1.10 on 2016-11-24 18:11
from __future__ import unicode_literals

from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0006_organization_organizationrole'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='is_superuser',
            field=djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Superusu\xe1rio?'),
        ),
        migrations.AlterField(
            model_name='user',
            name='password',
            field=djangoplus.db.models.fields.CharField(max_length=128, verbose_name='Senha'),
        ),
    ]



import datetime

from djangoplus.ui import Component
from djangoplus.utils.metadata import get_metadata
from djangoplus.utils.serialization import dumps_qs_query


class Calendar(Component):
    def __init__(self, request, title):
        super(Calendar, self).__init__(request)
        self.title = title
        self.items = []
        self.id = hash(self)
        self.lazy = False
        self.default_date = datetime.date.today()
        session_default_date = request.session.get('calendar_default_date')
        if session_default_date:
            self.default_date = datetime.datetime.strptime(session_default_date, '%Y-%m-%d')

    def add(self, description, start, end=None, url=None, allday=True):
        item = dict(description=description, start=start, end=end, allday=allday)
        self.items.append(item)

    def __unicode__(self):
        return self.render('calendar.html')


class ModelCalendar(Calendar):
    def __init__(self, request, title, editable=False):
        super(ModelCalendar, self).__init__(request, title)
        self.links = []
        self.lazy = True
        self.editable = editable

    def add(self, queryset, start_field, end_field=None, color='', action_names=[]):
        item = dict(queryset=dumps_qs_query(queryset), start_field=start_field, end_field=end_field, color=color, action_names=','.join(action_names))
        self.items.append(item)
        label = get_metadata(queryset.model, 'verbose_name')
        app_label = get_metadata(queryset.model, 'app_label')
        model_name = queryset.model.__name__.lower()
        url = '/add/{}/{}/?{}='.format(app_label, model_name, start_field)
        link = dict(label=label, url=url)
        self.links.append(link)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.forms import widgets
from django.utils.safestring import mark_safe
from djangoplus.utils.formatter import format_value


class DateWidget(widgets.DateInput):

    class Media:
        css = {'all': ('/static/css/daterangepicker-2.1.24.css',)}
        js = ('/static/js/moment.min.js', '/static/js/daterangepicker-2.1.24.js', '/static/js/jquery.mask.min-1.7.7.js')

    def render(self, name, value, attrs=None):
        attrs['class'] = 'form-control'
        html = super(DateWidget, self).render(name, value, attrs)
        script = '''
            $("#id_{}").daterangepicker({{
                singleDatePicker: true,
                showDropdowns: false,
                autoUpdateInput: false,
                locale: {{
                    format: 'DD/MM/YYYY'
                }}
                }});
            $("#id_{}").on('apply.daterangepicker', function(ev, picker) {{
                  $(this).val(picker.startDate.format('DD/MM/YYYY'));
              }});
            $("#id_{}").mask("00/00/0000", {{clearIfNotMatch: true}});
        '''.format(name, name, name)
        html = '<div class="input-group">{}<span class="input-group-addon"><i class="fa fa-calendar"></i></span></div><script>{}</script>'.format(html, script)
        return mark_safe(html)

    def _format_value(self, value):
        if value:
            if type(value) in [unicode, str]:
                return value
            else:
                return format_value(value)
        return ''


class HiddenDateWidget(widgets.DateInput):

    def render(self, name, value, attrs=None):
        attrs['class'] = 'hidden'
        return super(HiddenDateWidget, self).render(name, value, attrs)


class DateTimeWidget(widgets.DateTimeInput):

    class Media:
        css = {'all': ('/static/css/daterangepicker-2.1.24.css', )}
        js = ('/static/js/moment.min.js', '/static/js/daterangepicker-2.1.24.js', '/static/js/jquery.mask.min-1.7.7.js')

    def render(self, name, value, attrs=None):
        attrs['class'] = 'form-control'
        html = super(DateTimeWidget, self).render(name, value, attrs)
        if type(value) not in [unicode, str]:
            value = value and 'new Date{}'.format((value.year, value.month, value.day, value.hour, value.minute),) or 'new Date()'
        script = '''
                    $("#id_{}").daterangepicker({{
                        singleDatePicker: true,
                        showDropdowns: false,
                        autoUpdateInput: false,
                        startDate: {},
                        timePicker: true,
                        timePickerIncrement: 5,
                        timePicker24Hour: true,
                        locale: {{
                            format: 'DD/MM/YYYY H:mm',
                            cancelLabel: 'Cancelar',
                            applyLabel: 'Aplicar',
                        }}
                        }});
                        $("#id_{}").on('apply.daterangepicker', function(ev, picker) {{
                          $(this).val(picker.startDate.format('DD/MM/YYYY H:mm'));
                        }});
                        $("#id_{}").mask("00/00/0000 00:00", {{clearIfNotMatch: true}});
                '''.format(name, value, name, name)
        html = '<div class="input-group">{}<span class="input-group-addon"><i class="fa fa-calendar"></i></span></div><script>{}</script>'.format(
        html, script)
        return mark_safe(html)

    def _format_value(self, value):
        if type(value) in [unicode, str]:
            return value
        else:
            return format_value(value)


class DateRangeWidget(widgets.MultiWidget):
    def __init__(self, widgets=[DateWidget, DateWidget], attrs={}):
        super(DateRangeWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if not value:
            return ['', '']
        return value

    def render(self, name, value, attrs=None):
        if not isinstance(value, list):
            value = self.decompress(value)
        output = []
        final_attrs = self.build_attrs(attrs)
        id_ = final_attrs.get('id', None)
        output.append('<div class="s12" style="margin-left:-10px"><div class="col s6">A partir de')
        for i, widget in enumerate(self.widgets):
            try:
                widget_value = value[i]
            except IndexError:
                widget_value = None
            if id_:
                final_attrs = dict(final_attrs, id='{}_{}'.format(id_, i))
            output.append(widget.render(name + '_{}'.format(i), widget_value, final_attrs))
            if i == 0:
                output.append('</div><div class="col s6">Até')
        output.append('</div></div>')
        return mark_safe(self.format_output(output))


class DateFilterWidget(DateRangeWidget):

    class Media:
        css = {'all': ('/static/css/daterangepicker-2.1.24.css',)}
        js = ('/static/js/moment.min.js', '/static/js/daterangepicker-2.1.24.js', '/static/js/jquery.mask.min-1.7.7.js')

    def __init__(self, widgets=(HiddenDateWidget, HiddenDateWidget), attrs={}):
        super(DateRangeWidget, self).__init__(widgets, attrs)
        self.label = ''

    def decompress(self, value):
        if not value:
            return ['', '']
        return value

    def render(self, name, value, attrs=None):
        if not isinstance(value, list):
            value = self.decompress(value)
        output = []
        final_attrs = self.build_attrs(attrs)
        id_ = final_attrs.get('id', None)
        start = value and len(value) > 0 and value[0] or None
        end = value and len(value) > 1 and value[1] or None

        start_js = start and 'var start = moment("{}", "DD/MM/YYYY");'.format(start) or 'var start = null;'
        end_js = end and 'var end = moment("{}", "DD/MM/YYYY");'.format(end) or 'var end = null;'

        final_attrs.update(id='{}_0'.format(id_))
        final_attrs.update(**{'data-type':'hidden_daterange'})
        output.append(self.widgets[0].render('{}_0'.format(name), start, final_attrs))
        final_attrs.update(id='{}_1'.format(id_))
        output.append(self.widgets[1].render('{}_1'.format(name), end, final_attrs))
        function_name = name.replace('-', '_')
        script = '''
            <script>

                {}
                {}

                function cb{}(start, end, onload) {{
                    var s = "{}";
                    var startInput = $("#{}_0");
                    var endInput = $("#{}_1");
                    var format = "D [de] MMMM [de] YYYY";

                    if(start !=null && end !=null){{
                        s = start.format(format) + ' - ' + end.format(format);
                        startInput.val(start.format("DD/MM/YYYY"));;
                        endInput.val(end.format("DD/MM/YYYY"));
                    }}
                    else if (start!= null){{
                        s = ' A partir de  ' + start.format(format);
                        startInput.val(start.format("DD/MM/YYYY"));
                    }}
                    else if(end!=null){{
                        s = ' Até ' + end.format(format);
                        endInput.val(end.format("DD/MM/YYYY"));
                    }} else {{
                        $('#reportrange{} span').html(s);
                        startInput.val('');
                        endInput.val('');
                    }}

                    if(start !=null || end !=null) $('#clear{}').show();
                    else $('#clear{}').hide();

                    $('#reportrange{} span').html(s);

                    if(onload!=true){{
                        startInput.trigger('change');
                    }}
                }}

                $('#reportrange{}').daterangepicker({{
                    showCustomRangeLabel: false,
                    locale: {{format: "DD/MM/YYYY", applyLabel: "Selecionar", cancelLabel: "Cancelar"}},
                    linkedCalendars: false,
                    alwaysShowCalendars: true,
                    opens: "center",
                    ranges: {{
                       'Hoje': [moment(), moment().add(1, 'days')],
                       'Ontem': [moment().subtract(1, 'days'), moment().subtract(1, 'days')],
                       'Últimos 7 dias': [moment().subtract(6, 'days'), moment()],
                       'Últimos 30 dias': [moment().subtract(29, 'days'), moment()],
                       'Este mês': [moment().startOf('month'), moment().endOf('month')],
                       'Mês passado': [moment().subtract(1, 'month').startOf('month'), moment().subtract(1, 'month').endOf('month')]
                    }}
                }}, cb{});

                cb{}(start, end, true);

            </script>
        '''.format(start_js, end_js, function_name, self.label, id_, id_, name, name, name, name, name, function_name, function_name)
        output.append('''
            <div class="form-control" style="background: #fff; cursor: pointer; padding: 5px; height:34px; border: 1px solid #ccc; width: auto; margin-right:5px;">
                <span id="reportrange{}">
                <i class="fa fa-calendar"></i>&nbsp;&nbsp;
                    <span style="white-space: nowrap"></span>&nbsp;
                    </span><i id="clear{}" onclick="cb{}(null, null)" class="fa fa-close">
                </i>&nbsp;
                <b class="caret"></b>&nbsp;&nbsp;&nbsp;</div>'''.format(name, name, function_name))
        output.append(script)

        s = mark_safe(''.join(output))
        return s




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import zlib
import base64
import cPickle
from django.apps import apps

def dumps_qs_query(qs):
    query = base64.b64encode(zlib.compress(cPickle.dumps(qs.query)))[::-1]
    return '{}:::{}:::{}'.format(qs.model._meta.app_label, qs.model.__name__, query)


def loads_qs_query(s):
    app_label, model_name, query = s.split(':::')
    query = cPickle.loads(zlib.decompress(base64.b64decode(query[::-1])))
    qs = apps.get_model(app_label, model_name).objects.all()
    qs.query = query
    return qs


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import copy
from django.conf import settings
from djangoplus.cache import loader
from djangoplus.ui.components.menu import Menu
from djangoplus.admin.models import Settings
from djangoplus.utils import permissions
from djangoplus.utils.metadata import get_metadata


def context_processor(request):
    app_settings = Settings.default()
    alerts = []
    menu = None

    if request.user.is_authenticated:
        menu = Menu(request, app_settings)
        menu.load()

        for model in loader.subsets:
            icon = get_metadata(model, 'icon', 'fa-warning')
            title = get_metadata(model, 'verbose_name_plural')
            app_label = get_metadata(model, 'app_label')
            model_name = model.__name__.lower()
            for item in loader.subsets[model]:
                can_view = item['can_view']
                alert = item['alert']
                if alert and permissions.check_group_or_permission(request, can_view):
                    attr_name = item['function'].im_func.func_name
                    qs = model.objects.all(request.user)
                    qs = getattr(qs, attr_name)()
                    count = qs.count()
                    if count:
                        url = '/list/{}/{}/{}/'.format(app_label, model_name, attr_name)
                        item = dict(title=title, description=item['title'], count=count, url=url, icon=icon)
                        alerts.append(item)

    return dict(
        js_files=settings.EXTRA_JS,
        css_files=settings.EXTRA_CSS,
        settings=app_settings,
        menu=menu,
        alerts=alerts
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import json
from os import path
from django.conf import settings
from djangoplus.test import cache
from djangoplus.docs.doc import Workflow, UseCase
from django.core.management.commands import test


class Command(test.Command):

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--continue', action='store_true', dest='continue', default=False,
                            help='Continues from the last successfull login')
        parser.add_argument('--generate', action='store_true', dest='generate', default=(),
                            help='Adds test cases in test.py file')
        parser.add_argument('--watch', action='store_true', dest='watch', default=False,
                            help='Run test in browser intead of headless mode')

    def handle(self, *args, **options):
        if not options.pop('watch', False):
            os.environ['HEADLESS'] = '1'
        if options.pop('generate', False):
            workflow = Workflow()
            function_definitions = []
            function_calls = []
            test_file_path = path.join(settings.BASE_DIR, settings.PROJECT_NAME, 'tests.py')
            file_content = open(test_file_path).read().decode('utf-8')
            try:
                for task in workflow.tasks:
                    usecase = UseCase(task)
                    function_calls.append('\n'.join(usecase.get_test_flow_code()))
                    if usecase.get_test_function_code():
                        function_definitions.append('\n'.join(usecase.get_test_function_code()))
                function_definitions_code = '\n\n'.join(function_definitions)
                file_content = file_content.replace('(TestCase):', '(TestCase):\n\n{}'.format(function_definitions_code), 1)
                file_content = '{}\n{}'.format(file_content, '\n'.join(function_calls))
                file_content = file_content.replace('\t', '    ')
                open(test_file_path, 'w').write(file_content.encode('utf-8'))
            except ZeroDivisionError, e:
                print e

        else:
            if options.pop('continue', False):
                file_path = '/tmp/{}.test'.format(settings.PROJECT_NAME)
                file_content = path.exists(file_path) and open(file_path).read() or ''
                if file_content:
                    data = json.loads(file_content)
                    cache.LOGIN_COUNT = data.get('login_count', None)
                    username = data.get('username', None)
                    password = data.get('password', None)
                    print 'Continuing from login #{} with user "{}" and password "{}"'.format(cache.LOGIN_COUNT, username, password)

            super(Command, self).handle(*args, **options)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.template.loader import render_to_string
import random


class Component(object):

    def __init__(self, request=None):
        self.id = self.id = abs(random.randint(0, 900))
        self.request = request
        self.response = None
        self.as_pdf = False

    def render(self, template_name):
        self.as_pdf = self.request and self.request.GET.get('pdf', False) or self.as_pdf
        return render_to_string(template_name, {'self': self})



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.core.management.base import BaseCommand
from djangoplus.cache import loader


class Command(BaseCommand):
    def handle(self, *args, **options):
        for model in loader.role_models:
            for obj in model.objects.all():
                obj.save()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf import settings
from django.template import loader
from django.core.mail import EmailMultiAlternatives


def send_mail(subject, dictionary, template_name, to, reply_to=None):
    reply_to = reply_to and [reply_to] or None
    from_email = 'Não-Responder <{}>'.format(settings.SERVER_EMAIL)
    template_name = 'html' in template_name and template_name or '{}.html'.format(template_name)
    html = loader.render_to_string(template_name, dictionary)
    email = EmailMultiAlternatives(subject, 'Mensagem em anexo.', from_email, [to], reply_to=reply_to)
    email.attach_alternative(html, "text/html")
    return email.send()

# from djangoplus.utils.mail import send_mail;send_mail('oi', dict(), 'email_notification', 'brenokcc@yahoo.com.br')




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# Generated by Django 1.10 on 2016-09-10 10:09
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('admin', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('operation', djangoplus.db.models.fields.IntegerField(choices=[[1, 'Cadastro'], [2, 'Edi\xe7\xe3o'], [3, 'Exclus\xe3o']], verbose_name='Opera\xe7\xe3o')),
                ('date', djangoplus.db.models.fields.DateTimeField(auto_now=True, verbose_name='Data/Hora')),
                ('object_id', djangoplus.db.models.fields.IntegerField(verbose_name='Identificador')),
                ('object_description', djangoplus.db.models.fields.CharField(max_length=255, verbose_name='Descri\xe7\xe3o do Objeto')),
                ('content', djangoplus.db.models.fields.TextField(null=True, verbose_name='Conte\xfado')),
                ('content_type', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Objeto')),
                ('user', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Log',
                'verbose_name_plural': 'Logs',
            },
        ),
        migrations.CreateModel(
            name='LogIndex',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', djangoplus.db.models.fields.IntegerField(verbose_name='Identificador')),
                ('content_type', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Dado')),
                ('log', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='admin.Log', verbose_name='Log')),
            ],
            options={
                'verbose_name': 'Index',
                'verbose_name_plural': 'Indexes',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# Generated by Django 1.10 on 2016-10-09 23:50
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0005_auto_20161005_1536'),
    ]

    operations = [
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ascii', djangoplus.db.models.fields.SearchField(blank=True, default=b'', editable=False)),
            ],
            options={
                'verbose_name': 'Organiza\xe7\xe3o',
                'verbose_name_plural': 'Organiza\xe7\xf5es',
            },
        ),
        migrations.CreateModel(
            name='OrganizationRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('organization', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='admin.Organization', verbose_name='Organiza\xe7\xe3o')),
                ('role', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='admin.Role', verbose_name='Fun\xe7\xe3o')),
            ],
            options={
                'verbose_name': 'Papel na Organiza\xe7\xe3o',
                'verbose_name_plural': 'Papeis na Organiza\xe7\xe3o',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


def log_tags(value):
    l = []
    for item in value:
        if len(item) == 3:
            l.append('<span class="tag"><span>{}:&nbsp;&nbsp;</span>{} <i class="fa fa-arrow-right" aria-hidden="true"></i> {}</span>'.format(item[0], item[1], item[2]))
        else:
            l.append('<span class="tag"><span>{}:&nbsp;&nbsp;</span>{}</span>'.format(item[0], item[1]))
    return ''.join(l)


def progress(percentual):
    percentual = int(percentual)
    return '<div class="progress" data-toggle="tooltip" data-placement="top" title="" data-original-title="{}%"><div class="progress-bar" style="width: {}%;"></div></div>'.format(percentual, percentual)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import datetime, json
from django.http.response import HttpResponse
from djangoplus.templatetags import obj_icons
from djangoplus.ui.components.dropdown import ModelDropDown
from django.views.decorators.csrf import csrf_exempt
from djangoplus.utils.serialization import loads_qs_query


@csrf_exempt
def populate(request):
    items = []
    queryset = loads_qs_query(request.POST['queryset'])
    start_field = request.POST['start_field']
    end_field = request.POST['end_field']
    start = datetime.datetime.strptime(request.POST['start'], '%Y-%m-%d')
    end = datetime.datetime.strptime(request.POST['end'], '%Y-%m-%d')
    action_names = request.POST['action_names'].split(',')

    request.session['calendar_default_date'] = (start + datetime.timedelta(days=10)).strftime('%Y-%m-%d')
    request.session.save()

    filters = dict()
    filters['{}__gte'.format(start_field)] = start
    filters['{}__lte'.format(start_field)] = end
    queryset = queryset.filter(**filters)

    for obj in queryset.all():
        title = unicode(obj)
        start = getattr(obj, start_field)
        end = end_field and getattr(obj, end_field) or None
        if end:
            end= end + datetime.timedelta(days=1)
        icons = obj_icons(request, obj, css='popup')
        drop_down = ModelDropDown(request, obj.__class__, action_names=action_names)
        drop_down.add_actions(obj, inline=True)
        html_id = hash(obj)
        html = list()
        html.append('<div id="{}">'.format(html_id))
        html.append('<div class="pull-right">{}</div><br/><br/>'.format(icons))
        html.append('<dl>')
        html.append('<dt>Início</dt><dd>{}</dd>'.format(start))
        if end:
            html.append('<dt>Fim</dt><dd>{}</dd>'.format(end))
        html.append('</dl>')
        html.append('<hr/>')
        html.append(unicode(drop_down))
        html.append('</div>')
        html.append('<script>initialize("{}");</script>'.format(html_id))
        item = dict(id=html_id, title=title, start=str(start), end=end and str(end) or None, allDay=False, url='javascript:', html=''.join(html))
        items.append(item)

    return HttpResponse(json.dumps(items))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
from dropbox import Dropbox
from django.conf import settings
from dropbox.files import WriteMode
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        dropbox_destination = settings.PROJECT_NAME
        client = Dropbox(settings.DROPBOX_TOKEN)
        local_paths = []
        for relative_path in settings.BACKUP_FILES:
            file_or_directory = os.path.join(settings.BASE_DIR, relative_path)
            if os.path.isdir(file_or_directory):
                for root, dirs, files in os.walk(file_or_directory):
                    for filename in files:
                        local_path = os.path.join(root, filename)
                        relative_path = os.path.relpath(local_path, settings.BASE_DIR)
                        dropbox_path = os.path.join(dropbox_destination, relative_path)
                        local_paths.append((local_path, dropbox_path))
            else:
                dropbox_path = os.path.join(dropbox_destination, relative_path)
                local_paths.append((file_or_directory, dropbox_path))

        for local_path, dropbox_path in local_paths:
            dropbox_path = '/{}'.format(dropbox_path)
            print 'Uploading {} to {}...'.format(local_path, dropbox_path)
            with open(local_path, 'rb') as f:
                client.files_upload(f, dropbox_path, WriteMode.overwrite)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
function_views = []
widget_views = []
count = 0


def next_number():
    global count
    if count > 1000:
        count = 0
    count = count + 1
    return count



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import traceback, copy
from django.apps import apps
from django.conf import settings
from django.shortcuts import render
from djangoplus.cache import loader
from django.http import HttpResponse
from djangoplus.utils import permissions
from djangoplus.ui.components.panel import ModelPanel
from djangoplus.ui.components.breadcrumbs import httprr
from django.template import Template, Context
from djangoplus.ui.components.paginator import Paginator
from djangoplus.utils.http import ReportResponse
from django.views.defaults import page_not_found, server_error
from django.core.exceptions import ValidationError
from django.http.response import HttpResponseForbidden
from django.contrib.contenttypes.models import ContentType
from djangoplus.utils.metadata import list_related_objects, \
    is_many_to_many, is_one_to_one, get_metadata, check_condition, is_one_to_many
from djangoplus.ui.components.forms import factory, DEFAULT_FORM_TITLE, DEFAULT_SUBMIT_LABEL, ModelChoiceField


def listt(request, app, cls, subset=None):

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')
    subsetp = None
    if subset:
        subset_func = getattr(_model.objects.all(), subset)
        can_view = subset_func._metadata['{}:can_view'.format(subset)]
    else:
        tid = request.GET.get('tid')
        subsetp = request.GET.get('tab{}'.format(tid))
        if tid and subsetp:
            subset_func = getattr(_model.objects.get_queryset(), subsetp)
            can_view = subset_func._metadata['{}:can_view'.format(subsetp)]
            if not permissions.check_group_or_permission(request, can_view):
                return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))
        else:
            permission = '{}.list_{}'.format(app, cls)
            if not request.user.has_perm(permission):
                return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    qs = _model.objects.all(request.user)
    list_subsets = subset and [subset] or None
    if subset:
        title = getattr(getattr(qs, subset), '_metadata')['{}:title'.format(subset)]

    else:
        title = '{}'.format(get_metadata(_model, 'verbose_name_plural'))

    paginator = Paginator(request, qs, title, list_subsets=list_subsets, is_list_view=True)
    response = paginator.get_response()
    if response:
        return response

    if _model in loader.class_actions:
        for group in loader.class_actions[_model]:
            for view_name in loader.class_actions[_model][group]:
                _action = loader.class_actions[_model][group][view_name]
                action_title = _action['title']
                action_message = _action['message']
                action_can_execute = _action['can_execute']
                action_input = _action['input']
                action_css = _action['css']
                action_condition = _action['condition']
                initial = _action['initial']
                choices = _action['choices']

                if subsetp:
                    if subsetp not in loader.subset_actions[_model] or view_name not in loader.subset_actions[_model][subsetp]:
                        continue
                else:
                    if subset not in loader.subset_actions[_model] or view_name not in loader.subset_actions[_model][subset]:
                        continue

                if permissions.check_group_or_permission(request, action_can_execute):
                    func = hasattr(qs, view_name) and getattr(qs, view_name) or None
                    if func:

                        char = '?' in request.get_full_path() and '&' or '?'
                        url = '{}{}{}'.format(request.get_full_path(), char, '{}='.format(view_name))

                        has_input = func.func_code.co_argcount > 1

                        if not has_input:
                            action_css = action_css.replace('popup', '')
                        paginator.add_subset_action(action_title, url, action_css, None, action_condition)

                        if view_name in request.GET:
                            ids = paginator.get_selected_ids()
                            if ids:
                                qs = paginator.get_queryset(paginate=False).filter(id__in=ids)
                            else:
                                qs = paginator.get_queryset(paginate=False)

                            func = getattr(qs, view_name)
                            redirect_url = None

                            if has_input:
                                form = factory.get_class_action_form(request, _model, _action, func)
                                paginator = ''
                                if form.is_valid():
                                    params = []
                                    for param in func.func_code.co_varnames[1:func.func_code.co_argcount]:
                                        if param in form.cleaned_data:
                                            params.append(form.cleaned_data[param])
                                    try:
                                        f_return = func(*params)
                                        redirect_url = '..'
                                    except ValidationError, e:
                                        form.add_error(None, unicode(e.message))
                                if not redirect_url:
                                    return render(request, 'default.html', locals())
                            else:
                                f_return = func()
                                redirect_url = '.'

                            if redirect_url:
                                request.GET._mutable = True
                                del request.GET['ids']
                                del request.GET[view_name]
                                request.GET._mutable = False
                            return httprr(request, redirect_url, action_message)
                    else:
                        url = '/{}/{}/'.format(app, view_name)
                        if view_name in request.GET:
                            return httprr(request, url)
                        else:
                            action_css = action_css.replace('popup', '')
                            paginator.add_action(action_title, url, action_css, None)

    paginator.add_actions()

    return render(request, 'default.html', locals())


def add(request, app, cls, pk=None, related_field_name=None, related_pk=None):

    if not request.user.is_authenticated():
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    obj = pk and _model.objects.all(request.user).get(pk=pk) or _model()
    obj.request = request

    title = pk and unicode(obj) or get_metadata(_model, 'verbose_name')

    if not related_field_name:

        if obj.pk:
            if not permissions.has_edit_permission(request, _model) or not permissions.can_edit(request, obj):
                return HttpResponseForbidden()
        else:
            if not permissions.has_add_permission(request, _model) or not permissions.can_add(request, obj):
                return HttpResponseForbidden()

        form = factory.get_register_form(request, obj)
        title = form.title
        
    elif is_one_to_many(_model, related_field_name):
        if not permissions.can_add(request, obj) and not permissions.can_edit(request, obj):
            return HttpResponseForbidden()
        form = factory.get_one_to_many_form(request, obj, related_field_name)

    elif is_many_to_many(_model, related_field_name):
        if not permissions.can_add(request, obj) and not permissions.can_edit(request, obj):
            return HttpResponseForbidden()
        form = factory.get_many_to_many_form(request, obj, related_field_name, related_pk)

    elif is_one_to_one(_model, related_field_name):
        if not permissions.can_add(request, obj) and not permissions.can_edit(request, obj):
            return HttpResponseForbidden()
        form = factory.get_one_to_one_form(request, obj, related_field_name, related_pk)
    else:
        # many to one
        for rel in list_related_objects(_model):
            if hasattr(rel, 'get_accessor_name'):
                if rel.get_accessor_name() in ('{}_set'.format(related_field_name), related_field_name):
                    related_queryset = rel.related_model.objects.all(request.user)
                    related_obj = related_pk and related_queryset.get(pk=related_pk) or rel.related_model()
                    related_obj.request = request
                    setattr(related_obj, rel.field.name, obj)
                    setattr(related_obj, '{}_id'.format(rel.field.name), obj.pk)
                    if related_pk:
                        if not permissions.has_edit_permission(request, rel.related_model) or not permissions.can_edit(request, related_obj):
                            return HttpResponseForbidden()
                    else:
                        if not permissions.has_add_permission(request, rel.related_model) or not permissions.can_add(request, related_obj):
                            return HttpResponseForbidden()

                    form = factory.get_many_to_one_form(request, obj, rel.get_accessor_name(), related_obj)
                    title = form.title

    if form.is_valid():
        is_editing = form.instance.pk is not None
        try:
            form.save()
            obj = form.instance
            if 'select' in request.GET:
                return HttpResponse('{}|{}|{}'.format(obj.pk, obj, request.GET['select']));
            elif related_field_name:
                message = 'Ação realizada com sucesso'
                url = '..'
            else:
                message = get_metadata(form.instance.__class__, 'add_message')
                if message and not is_editing:
                    if hasattr(obj, 'get_absolute_url'):
                        url = obj.get_absolute_url()
                    else:
                        url = '/view/{}/{}/{}/'.format(get_metadata(obj.__class__, 'app_label'), obj.__class__.__name__.lower(), obj.pk)
                else:
                    url = '..'
                if is_editing:
                    message = message or 'Atualização realizada com sucesso'
                else:
                    message = message or 'Cadastro realizado com sucesso'
            return httprr(request, url, message)
        except ValidationError, e:
            form.add_error(None, unicode(e.message))
    return render(request, 'default.html', locals())


def view(request, app, cls, pk, tab=None):

    if not request.user.is_authenticated():
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    qs = _model.objects.all(request.user)

    obj = qs.get(pk=pk)
    obj.request = request
    obj._user = request.user

    if not permissions.can_view(request, obj):
        return HttpResponseForbidden()

    title = unicode(obj)
    parent = request.GET.get('parent', None)
    panel = ModelPanel(request, obj, tab, parent)

    if request.GET.get('pdf', False):
        return ReportResponse(title, request, [panel])
    elif panel.message:
        return httprr(request, request.get_full_path(), panel.message)

    log_data = get_metadata(obj.__class__, 'log', False)
    if log_data and request.user.is_superuser and request.user.has_perm('admin.list_log'):
        url = '/log/{}/{}/'.format(app, cls)
        panel.drop_down.add_action('Visualizar Log', url, 'ajax', 'fa fa-history')

    return render(request, 'default.html', locals())


def action(request, app, cls, action_name, pk=None):

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    for group in loader.actions[_model]:
        if action_name in loader.actions[_model][group]:
            break

    form_action = loader.actions[_model][group][action_name]
    action_title = form_action['title']
    action_can_execute = form_action['can_execute']
    action_condition = form_action['condition']
    action_function = form_action['function']
    action_message = 'message' in form_action and form_action['message'] or None
    action_permission = '{}.{}'.format(_model._meta.app_label, action_function.func_name)
    action_input = form_action['input']
    redirect_to = form_action['redirect_to']

    obj = pk and _model.objects.all(request.user).get(pk=pk) or _model()
    obj.request = request
    obj._user = request.user
    title = action_title

    if check_condition(action_condition, obj) and (not action_can_execute or permissions.check_group_or_permission(request, action_permission)):

        func = getattr(_model, action_function.func_name, action_function)
        form = factory.get_action_form(request, obj, form_action)

        if func.func_code.co_argcount > 1 or action_input:
            if form.is_valid():
                if 'instance' in form.fields:
                    obj = form.cleaned_data['instance']
                func = getattr(obj, action_function.func_name, action_function)
                params = []
                for param in func.func_code.co_varnames[1:func.func_code.co_argcount]:
                    if param in form.cleaned_data:
                        params.append(form.cleaned_data[param])
                try:
                    f_return = func(*params)
                    if not redirect_to:
                        if func.func_code.co_argcount > 1:
                            return httprr(request, '..', action_message)
                        else:
                            return httprr(request, '.', action_message)
                    else:
                        return httprr(request, Template(redirect_to).render(Context({'self': obj})), action_message)
                except ValidationError, e:
                    form.add_error(None, unicode(e.message))
        else:
            try:
                if form.fields and form.is_valid() or not form.fields:
                    if form.fields:
                        obj = form.cleaned_data['instance']
                    func = getattr(obj, action_function.func_name, action_function)
                    f_return = func()
                    if not redirect_to:
                        if func.func_code.co_argcount > 1:
                            return httprr(request, '..', action_message)
                        else:
                            return httprr(request, '.', action_message)
                    else:
                        return httprr(request, Template(redirect_to).render(Context({'self': obj})), action_message)
            except ValidationError, e:
                return httprr(request, '.', unicode(e.message))

        if form.title == DEFAULT_FORM_TITLE:
            form.title = action_title
        if form.submit_label == DEFAULT_SUBMIT_LABEL:
            form.submit_label = action_title
        return render(request, 'default.html', locals())
    else:
        return HttpResponseForbidden()


def delete(request, app, cls, pk, related_field_name=None, related_pk=None):

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    obj = _model.objects.all(request.user).get(pk=pk)
    obj.request = request

    permission_name = '{}.delete_{}'.format(app, cls)
    if permissions.can_delete(request, obj) and permissions.check_group_or_permission(request, permission_name):

        if related_field_name:
            getattr(obj, related_field_name).remove(related_pk)
            return httprr(request, '..', 'Removido com sucesso')
        else:
            title = 'Excluir {}'.format(unicode(obj))
            form = factory.get_delete_form(request, obj)
            if form.is_valid():
                obj.delete()
                return httprr(request, '..', 'Exclusão realizada com sucesso.')

            return render(request, 'delete.html', locals())

    else:
        return HttpResponseForbidden()


def log(request, app, cls, pk=None):
    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    if pk:
        obj = _model.objects.get(pk=pk)
        qs = obj.get_logs()
        title = 'Log - {}'.format(obj)
    else:
        content_type = ContentType.objects.get_for_model(_model)
        qs = content_type.log_set.all()
        title = 'Logs - {}'.format(get_metadata(_model, 'verbose_name_plural'))

    paginator = Paginator(request, qs, 'Log')
    return render(request, 'default.html', locals())


def dispatcher(request, app, view_name, params):

    params = params.split('/')[0:-1]

    full_app_name = settings.APP_MAPPING.get(app, app)
    fromlist = full_app_name.split('.')

    try:
        views = __import__('{}.views'.format(full_app_name), fromlist=list(map(str, fromlist)))
        func = getattr(views, view_name)
    except (ImportError, TypeError, AttributeError) as e:
        traceback.print_exc()
        return page_not_found(request, e, 'error404.html')

    try:
        return func(request, *params)
    except Exception as e:
        print e
        traceback.print_exc()
        return server_error(request, 'error500.html')





# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.template.loader import render_to_string
from django.http import HttpResponse
from django.conf import settings
import tempfile
import json
import os
import datetime


def mobile(request):
    for device in ['iphone', 'android']:
        if device in request.META.get('HTTP_USER_AGENT', '').lower():
            return True
    return False


class XlsResponse(HttpResponse):
    def __init__(self, data, name='Listagem'):
        import xlwt
        import StringIO
        output = StringIO.StringIO()
        wb = xlwt.Workbook(encoding='iso8859-1')
        for title, rows in data:
            sheet = wb.add_sheet(title)
            for row_idx, row in enumerate(rows):
                for col_idx, label in enumerate(row):
                    sheet.write(row_idx, col_idx, label=label)
        wb.save(output)
        HttpResponse.__init__(self, content=output.getvalue(), content_type='application/vnd.ms-excel')
        self['Content-Disposition'] = 'attachment; filename={}.xls'.format(name)


class CsvResponse(HttpResponse):
    def __init__(self, rows, name='Listagem'):
        import unicodecsv
        import StringIO
        output = StringIO.StringIO()
        delimiter = os.sep == '/' and ',' or ';'
        writer = unicodecsv.writer(output, delimiter=delimiter, encoding='iso8859-1')
        for row in rows:
            writer.writerow(row)
        HttpResponse.__init__(self, content=output.getvalue(), content_type='application/csv')
        self['Content-Disposition'] = 'attachment; filename={}.xls'.format(name)


class ZipResponse(HttpResponse):
    def __init__(self, file_path):
        content = open(file_path)
        HttpResponse.__init__(self, content=content, content_type='application/zip')
        self['Content-Disposition'] = 'attachment; filename={}'.format(file_path.split(os.sep)[-1])


class JsonResponse(HttpResponse):
    def __init__(self, data):
        content = json.dumps(data)
        HttpResponse.__init__(self, content=content, content_type='application/json')


class PdfResponse(HttpResponse):

    def __init__(self, html, landscape=False):
        from xhtml2pdf import pisa

        def link_callback(uri, rel):
            s = '{}/{}'.format(settings.MEDIA_ROOT, uri)
            return s

        tmp = tempfile.NamedTemporaryFile(mode='w+b', delete=False)
        file_name = tmp.name
        if landscape:
            html = html.replace('a4 portrait', 'a4 landscape')
            html = html.replace('logo_if_portrait', 'logo_if_landscape')
        out = pisa.CreatePDF(html, tmp, link_callback=link_callback)
        out.dest.close()
        tmp = file(file_name, "rb")
        str_bytes = tmp.read()
        os.unlink(file_name)
        HttpResponse.__init__(self, str_bytes, content_type='application/pdf')


class ReportResponse(PdfResponse):
    def __init__(self, title, request, objects, landscape=False, template='report.html'):
        html = render_to_string([template], dict(objects=objects, title=title, today=datetime.date.today()), request=request)
        super(ReportResponse, self).__init__(html, landscape)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from fabric.api import *
from os import path

WORKSPACE_PATH = '/Users/breno/Documents/Workspace'
PYTHON_PATH = '/var/opt/.virtualenvs/djangoplus/bin/python'
PROJECT_NAMES = ['abstract', 'blackpoint', 'emprestimos', 'fabrica', 'financeiro', 'formulacao', 'gouveia', 'companies']
EMPTY_TEST_FILE_CONTENT = '''# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from djangoplus._test_admin.models import User
from djangoplus.test import TestCase
from django.conf import settings


class AppTestCase(TestCase):

    def test_app(self):

        User.objects.create_superuser('_test_admin', None, settings.DEFAULT_PASSWORD)

        self.login('_test_admin', settings.DEFAULT_PASSWORD)
'''


DOCKER_FILE_CONTENT = '''FROM {}

RUN apt-get update
RUN apt-get -y install python-pip build-essential python-dev libfreetype6-dev libtiff5-dev liblcms2-dev libwebp-dev tk8.6-dev libjpeg-dev ssh openssh-server dnsutils curl vim git
RUN apt-get -y install chrpath libssl-dev libxft-dev libfreetype6 libfreetype6-dev libfontconfig1 libfontconfig1-dev
RUN wget https://github.com/ariya/phantomjs/releases/download/2.1.3/phantomjs -P /usr/bin/
'''


def _test_start_project():
    if path.exists('/tmp/xxx'):
        local('rm -r /tmp/xxx')
    with lcd('/tmp/'):
        local('{}/djangoplus/djangoplus/bin/startproject xxx'.format(WORKSPACE_PATH))
        with lcd('/tmp/xxx'):
            local('{} manage.py test'.format(PYTHON_PATH))
        with lcd('/tmp'):
            local('rm -r /tmp/xxx')


def _test_project():
    local('{} manage.py test'.format(PYTHON_PATH))


def _test_admin():
    local('{} manage.py test djangoplus.admin.tests.AdminTestCase'.format(PYTHON_PATH))


def _test_workspace():
    with lcd(WORKSPACE_PATH):
        for test_project_name in PROJECT_NAMES:
            with lcd(test_project_name):
                local('{} manage.py test'.format(PYTHON_PATH))
    with lcd(path.join(WORKSPACE_PATH, 'djangoplus/djangoplus-demos')):
        for test_project_name in ['petshop', 'loja', 'biblioteca']:
            with lcd(test_project_name):
                local('{} manage.py test'.format(PYTHON_PATH))


def _test_testcases_generation():
    test_file_path = '{}/emprestimos/emprestimos/tests.py'.format(WORKSPACE_PATH)
    test_file_content = open(test_file_path).read()
    open(test_file_path, 'w').write(EMPTY_TEST_FILE_CONTENT)
    with lcd('{}/emprestimos'.format(WORKSPACE_PATH)):
        local('python manage.py test --add')
    print open(test_file_path).read()
    open(test_file_path, 'w').write(test_file_content)


def _test_so_installation(so):
    docker_file = open('/tmp/Dockerfile', 'w')
    docker_file.write(DOCKER_FILE_CONTENT.format(so))
    docker_file.close()
    local('docker build -t djangoplus-{} /tmp'.format(so))
    #local('docker run djangoplus-{} pip install djangoplus && startproject xyz && cd xyz && python manage.py test djangoplus.admin.tests.AdminTestCase'.format(so))


def test_installation():
    # _test_so_instalation('debian')
    _test_so_installation('ubuntu')


def testall():
    _test_start_project()
    _test_admin()
    _test_workspace()


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf.urls import url
from djangoplus.ui.components.breadcrumbs import views

urlpatterns = [
    url(r'^breadcrumbs/reset/(?P<path>.*)$', views.reset),
]


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from decimal import Decimal

from djangoplus.ui import Component
from django.utils.text import slugify
from djangoplus.ui.components.forms import ModelForm, ValidationError
from djangoplus.utils import permissions
from djangoplus.ui.components.paginator import Paginator
from djangoplus.ui.components.dropdown import ModelDropDown, GroupDropDown
from django.template.loader import render_to_string
from djangoplus.utils.metadata import get_metadata, list_related_objects,\
    find_field_by_name, get_fiendly_name, check_condition, is_one_to_one, is_one_to_many, is_many_to_many, \
    is_one_to_many_reverse, is_many_to_one, should_filter_or_display


class Panel(Component):

    def __init__(self, request, title=None, text=None):
        super(Panel, self).__init__(request)
        self.title = title
        self.text = text
        self.labels = []
        self.icon = None

    def add_label(self, description, css='label-info'):
        label = dict(description=description, css=css)
        self.labels.append(label)

    def set_icon(self, name, css='text-info', align='right'):
        self.icon = dict(name=name, css=css, align=align)

    def __unicode__(self):
        return self.render('panel.html')


class ModelPanel(Component):
    def __init__(self, request, obj, current_tab=None, parent=None, fieldsets=None, complete=True):

        super(ModelPanel, self).__init__(request=request)

        self.obj = obj
        self.request = request
        self.title = unicode(obj)
        self.tabs = []
        self.current_tab = current_tab
        self.message = None
        self.complete = complete
        self.drop_down = None
        fieldsets = fieldsets or get_metadata(type(obj), 'view_fieldsets', [])
        if not fieldsets:
            fields = []
            for field in get_metadata(type(obj), 'fields')[1:]:
                fields.append(field.name)

            for field in get_metadata(type(obj), 'local_many_to_many'):
                fields.append(field.name)

            fieldsets = (('Dados Gerais', dict(fields=fields)),)

        if self.complete:
            self.drop_down = ModelDropDown(self.request, type(self.obj))
            self.drop_down.add_actions(self.obj, fieldset_title='')
        else:
            self.drop_down = GroupDropDown(self.request)

        self.fieldsets_with_tab_name = []
        self.fieldsets_without_tab_name = []

        model = type(self.obj)
        obj.as_pdf = self.as_pdf

        for fieldset in fieldsets:
            title, info = fieldset
            tab_name = None

            drop_down = ModelDropDown(self.request, model)
            fieldset_actions = info.get('actions', [])

            if fieldset_actions:
                drop_down.add_actions(self.obj, fieldset_title=title)

            if 'condition' in fieldset[1]:
                condition = fieldset[1]['condition']
                self.obj.request = self.request
                if not check_condition(condition, self.obj):
                    continue

            if '::' in title:
                tab_name, title = title.split('::')
                url = '/view/{}/{}/{}/{}/'.format(get_metadata(model, 'app_label'), model.__name__.lower(), self.obj.pk, slugify(tab_name))
                tab = (tab_name, url)
                if not self.tabs and not self.current_tab:
                    self.current_tab = slugify(tab_name)
                if tab not in self.tabs:
                    self.tabs.append(tab)

            if not tab_name or slugify(tab_name) == self.current_tab or self.as_pdf:

                fieldset_dict = dict(title=title or 'Dados Gerais', tab_name=tab_name, fields=[], paginators=[], drop_down=drop_down, image=None)
                relations = fieldset[1].get('relations', [])

                if tab_name or self.as_pdf:
                    self.fieldsets_with_tab_name.append(fieldset_dict)
                else:
                    self.fieldsets_without_tab_name.append(fieldset_dict)

                if 'can_view' in fieldset[1]:
                    can_view = fieldset[1]['can_view']
                    if not permissions.check_group_or_permission(self.request, can_view):
                        continue

                if 'image' in fieldset[1]:
                    fieldset_dict['image'] = fieldset[1]['image']

                if 'fields' in fieldset[1]:
                    for name_or_tuple in fieldset[1]['fields']:

                        if not type(name_or_tuple) == tuple:
                            name_or_tuple = (name_or_tuple,)
                        attr_names = []

                        for attr_name in name_or_tuple:
                            if attr_name != parent:
                                attr = getattr(model, attr_name)
                                field = None
                                if hasattr(attr, 'field_name'):
                                    field = getattr(model, '_meta').get_field(attr.field_name)
                                elif hasattr(attr, 'field'):
                                    field = attr.field
                                if not field or not hasattr(field, 'display') or field.display:
                                    if is_one_to_one(model, attr_name) and hasattr(attr.field, 'display') and attr.field.display == 'detail':
                                        relations.append(attr_name)
                                    elif is_one_to_many(model, attr_name) and field.display == 'detail':
                                        relations.append(attr_name)
                                    elif is_many_to_one(model, attr_name) and field.display == 'detail':
                                        relations.append(attr_name)
                                    elif is_many_to_many(model, attr_name) and (not hasattr(field, 'display') or field.display == 'detail'):
                                        relations.append(attr_name)
                                    elif is_one_to_many_reverse(model, attr_name):
                                        relations.append(attr_name)
                                    else:
                                        verbose_name, lookup, sortable, to = get_fiendly_name(model, attr_name, as_tuple=True)
                                        if to and not should_filter_or_display(self.request, model, to):
                                            continue
                                        attr_names.append(dict(verbose_name=verbose_name, name=attr_name))
                        if attr_names:
                            fieldset_dict['fields'].append(attr_names)

                if self.complete:

                    for relation_name in relations:
                        if relation_name in [field.name for field in get_metadata(model, 'get_fields')]:
                            relation_field = find_field_by_name(model, relation_name)
                            relation = getattr(self.obj, relation_name)
                            if not relation and hasattr(relation_field, 'rel') and relation_field.rel.to:
                                relation = relation_field.rel.to()
                            if hasattr(relation.__class__, 'pk'):
                                if relation.pk:
                                    fieldset_title = relation_field.verbose_name
                                    panel_fieldsets = get_metadata(type(relation), 'view_fieldsets', [])
                                    panel_fieldsets = ((fieldset_title, panel_fieldsets[0][1]), )
                                    panel = ModelPanel(request, relation, fieldsets=panel_fieldsets, complete=False)

                                    if is_one_to_one(model, relation_name):
                                        app_label = get_metadata(model, 'app_label')
                                        model_name = model.__name__.lower()
                                        related_model_name = type(relation).__name__.lower()
                                        add_url = '/add/{}/{}/{}/{}'.format(app_label, model_name, self.obj.pk, relation_name)
                                        delete_url = None
                                        if relation.pk:
                                            add_url = '{}/{}/'.format(add_url, relation.pk)
                                            app_label = get_metadata(type(relation), 'app_label')
                                            delete_url = '/delete/{}/{}/{}/'.format(app_label, related_model_name, relation.pk)
                                        if permissions.has_add_permission(self.request, model) or permissions.has_edit_permission(self.request, model):
                                            if delete_url:
                                                panel.drop_down.add_action('Excluir {}'.format(relation_field.verbose_name),
                                                                           delete_url, 'popup', 'fa-close', None)
                                            panel.drop_down.add_action('Atualizar {}'.format(relation_field.verbose_name),
                                                                       add_url, 'popup', 'fa-edit')

                                    fieldset_dict['paginators'].append(panel)
                            else:
                                fieldset_title = len(relations) > 1 and title or relation_field.verbose_name

                                if is_one_to_many(model, relation_name) or is_many_to_many(model, relation_name):
                                    to = model.__name__.lower()
                                else:
                                    to = relation_name

                                related_paginator = Paginator(self.request, relation.all(), title=fieldset_title, to=to, list_subsets=[])

                                add_url = '/add/{}/{}/{}/{}/'.format(get_metadata(model, 'app_label'), model.__name__.lower(), self.obj.pk, relation_name)
                                if permissions.has_add_permission(self.request, model) or permissions.has_relate_permission(self.request, model):
                                    related_paginator.add_action('Adicionar {}'.format(unicode(get_metadata(relation.model, 'verbose_name'))),
                                                                 add_url, 'popup', 'fa-plus')
                                fieldset_dict['paginators'].append(related_paginator)
                        else:
                            is_object_set = False
                            for related_object in list_related_objects(model):
                                if relation_name == related_object.get_accessor_name():
                                    is_object_set = True
                                    break
                            relation = getattr(self.obj, relation_name)
                            if hasattr(relation, 'all'):
                                qs = relation.all()
                            elif hasattr(relation, '__call__'):
                                qs = relation()
                            else:
                                qs = relation
                            to = is_object_set and related_object.field.name or None

                            fieldset_title = len(relations) > 1 and get_metadata(qs.model, 'verbose_name_plural') or title

                            if hasattr(relation, '_metadata'):
                                fieldset_title = relation._metadata['{}:verbose_name'.format(relation_name)]

                            exclude = [is_object_set and related_object.field.name or '']

                            related_paginator = Paginator(self.request, qs, fieldset_title, exclude=exclude, list_subsets=[], to=to, readonly=not is_object_set)
                            if is_object_set and (permissions.has_add_permission(self.request, qs.model) or permissions.has_relate_permission(self.request, qs.model)):
                                instance = qs.model()
                                setattr(instance, related_object.field.name, self.obj)
                                if permissions.can_add(self.request, instance):
                                    add_inline = get_metadata(qs.model, 'add_inline')
                                    if add_inline:
                                        form_name = get_metadata(qs.model, 'add_form')
                                        if form_name:
                                            fromlist = get_metadata(qs.model, 'app_label')
                                            forms_module = __import__('{}.forms'.format(fromlist), fromlist=list(map(str, [fromlist])))
                                            Form = getattr(forms_module, form_name)
                                        else:
                                            class Form(ModelForm):
                                                class Meta:
                                                    model = qs.model
                                                    fields = get_metadata(qs.model, 'form_fields', '__all__')
                                                    exclude = get_metadata(qs.model, 'exclude_fields', ())
                                                    submit_label = 'Adicionar'
                                                    title = 'Adicionar {}'.format(get_metadata(qs.model, 'verbose_name'))
                                        form = Form(self.request, instance=instance, inline=True)
                                        if related_object.field.name in form.fields:
                                            del(form.fields[related_object.field.name])
                                        related_paginator.form = form
                                        if form.is_valid():
                                            try:
                                                form.save()
                                                self.message = 'Ação realizada com sucesso'
                                            except ValidationError, e:
                                                form.add_error(None, unicode(e.message))
                                    else:
                                        add_url = '/add/{}/{}/{}/{}/'.format(
                                        get_metadata(model, 'app_label'), model.__name__.lower(), self.obj.pk,
                                        relation_name.replace('_set', ''))
                                        add_label = 'Adicionar {}'.format(get_metadata(qs.model, 'verbose_name'))
                                        add_label = get_metadata(qs.model, 'add_label', add_label)
                                        related_paginator.add_action(add_label, add_url,
                                                                     'popup', 'fa-plus')

                            fieldset_dict['paginators'].append(related_paginator)

                if 'extra' in fieldset[1]:
                    fieldset_dict['extra'] = []
                    for info in fieldset[1]['extra']:
                        fieldset_dict['extra'].append(info)

    def get_active_fieldsets(self):
        return self.fieldsets_without_tab_name + self.fieldsets_with_tab_name

    def __unicode__(self):
        return self.render('model_panel.html')


class IconPanel(Component):

    def __init__(self, request):
        from djangoplus.cache import loader
        super(IconPanel, self).__init__(request)
        self.items = []

        for model, add_shortcut in loader.icon_panel_models:

            if type(add_shortcut) == bool:
                add_model = add_shortcut
            else:
                if not type(add_shortcut) in (list, tuple):
                    add_shortcut = add_shortcut,
                add_model = request.user.in_group(*add_shortcut)

            if add_model or request.user.is_superuser:
                if permissions.has_add_permission(request, model):
                    self.add_model(model)
        for item in loader.views:
            if item['add_shortcut']:
                if permissions.check_group_or_permission(request, item['can_view']):
                    self.add(item['icon'], item['title'], None, item['url'], item['can_view'], item['style'])

    def add(self, icon, description, count=None, url=None, perm_or_group=None, style='ajax'):
        if permissions.check_group_or_permission(self.request, perm_or_group):
            item = dict(icon=icon, description=description, count=count, url=url or '#', style=style)
            self.items.append(item)

    def __unicode__(self):
        return self.render('icon_panel.html')

    def add_model(self, model):
        icon = get_metadata(model, 'icon')
        prefix = get_metadata(model, 'verbose_female') and 'Nova' or 'Novo'
        description = '{} {}'.format(prefix, get_metadata(model, 'verbose_name'))
        description = get_metadata(model, 'add_label', description)
        url = '/add/{}/{}/'.format(get_metadata(model, 'app_label'), model.__name__.lower())
        permission = '{}.list_{}'.format(get_metadata(model, 'app_label'), model.__name__.lower())
        self.add(icon, description, None, url, permission)

    def add_models(self, *models):
        for model in models:
            self.add_model(model)


class CardPanel(Component):

    def __init__(self, request):
        from djangoplus.cache import loader
        super(CardPanel, self).__init__(request)
        self.items = []

        for model, list_shortcut in loader.card_panel_models:

            if type(list_shortcut) == bool:
                add_model = list_shortcut
            else:
                if not type(list_shortcut) in (list, tuple):
                    list_shortcut = list_shortcut,
                add_model = request.user.in_group(*list_shortcut)

            if add_model or request.user.is_superuser:
                if permissions.has_list_permission(request, model):
                    self.add_model(model)

                if model in loader.subsets:
                    icon = get_metadata(model, 'icon')
                    title = get_metadata(model, 'verbose_name_plural')
                    app_label = get_metadata(model, 'app_label')
                    model_name = model.__name__.lower()
                    for item in loader.subsets[model]:
                        can_view = item['can_view']
                        # TODO False
                        if False and permissions.check_group_or_permission(self.request, can_view):
                            attr_name = item['function'].im_func.func_name
                            qs = model.objects.all(self.request.user)
                            qs = getattr(qs, attr_name)()
                            count = qs.count()
                            if count:
                                url = '/list/{}/{}/{}/'.format(app_label, model_name, attr_name)
                                self.add(icon, title, count, url, 'bg-info', None, item['title'])

        for item in loader.views:
            if False:  # TODO False
                if permissions.check_group_or_permission(request, item['can_view']):
                    self.add(item['icon'], item['menu'], None, item['url'], 'bg-info', item['can_view'], item['style'])

    def add(self, icon, title, count=None, url=None, css='bg-info', perm_or_group=None, description=''):
        if permissions.check_group_or_permission(self.request, perm_or_group):
            item = dict(icon=icon, title=title, count=count, url=url, css=css, description=description)
            self.items.append(item)

    def __unicode__(self):
        return self.render('card_panel.html')

    def add_model(self, model):
        self.add_models(model)

    def add_models(self, *models):
        for model in models:
            icon = get_metadata(model, 'icon')
            title = get_metadata(model, 'verbose_name_plural')
            app_label = get_metadata(model, 'app_label')
            model_name = model.__name__.lower()
            url = '/list/{}/{}/'.format(app_label, model_name)
            permission = '{}.list_{}'.format(app_label, model_name)
            self.add(icon, title, None, url, 'bg-info', permission)


class DashboardPanel(Component):

    def __init__(self, request):
        from djangoplus.cache import loader
        super(DashboardPanel, self).__init__(request)
        self.top = []
        self.center = []
        self.left = []
        self.right = []
        self.bottom = []

        from djangoplus.cache import loader
        for model in loader.subsets:
            icon = get_metadata(model, 'icon', 'fa-bell-o')
            title = get_metadata(model, 'verbose_name_plural')
            app_label = get_metadata(model, 'app_label')
            model_name = model.__name__.lower()
            for item in loader.subsets[model]:
                description = item['title']
                can_view = item['can_view']
                notify = item['notify']
                if notify and permissions.check_group_or_permission(request, can_view):
                    attr_name = item['function'].im_func.func_name
                    qs = model.objects.all(request.user)
                    qs = getattr(qs, attr_name)()
                    count = qs.count()
                    if count:
                        url = '/list/{}/{}/{}/'.format(app_label, model_name, attr_name)
                        notification_panel = NotificationPanel(request, title, count, url, description, icon)
                        self.right.append(notification_panel)

        icon_panel = IconPanel(request)
        card_panel = CardPanel(request)

        self.top.append(icon_panel)
        self.center.append(card_panel)

        for item in loader.widgets:
            if permissions.check_group_or_permission(request, item['can_view']):
                function = item['function']
                position = item['position']
                f_return = function(request)
                html = render_to_string(['{}.html'.format(function.func_name), 'widget.html'], f_return, request)
                if position == 'top':
                    self.top.append(html)
                elif position == 'center':
                    self.center.append(html)
                elif position == 'left':
                    self.left.append(html)
                elif position == 'right':
                    self.right.append(html)
                elif position == 'bottom':
                    self.bottom.append(html)
        for item in loader.subset_widgets:
            if permissions.check_group_or_permission(request, item['can_view']):
                model = item['model']
                title = item['title']
                function = item['function']
                position = item['position']
                formatter = item['formatter']
                qs = model.objects.all()
                qs.user = request.user
                f_return = getattr(qs, function)()
                html = ''

                if type(f_return) in (int, Decimal):
                    verbose_name = get_metadata(model, 'verbose_name_plural')
                    icon = get_metadata(model, 'icon')
                    panel = NumberPanel(request, verbose_name, f_return, title, icon)
                    html = unicode(panel)

                if type(f_return).__name__ == 'QueryStatistics' and not formatter:
                    formatter = 'statistics'

                if formatter:
                    func = loader.formatters[formatter]
                    if len(func.func_code.co_varnames) == 1:
                        html = unicode(func(f_return))
                    else:
                        html = unicode(func(f_return, request=self.request, verbose_name=title))
                elif hasattr(f_return, 'model'):
                    paginator = Paginator(self.request, f_return, title, readonly=True, list_filter=(), search_fields=())
                    html = unicode(paginator)

                if position == 'top':
                    self.top.append(html)
                elif position == 'center':
                    self.center.append(html)
                elif position == 'left':
                    self.left.append(html)
                elif position == 'right':
                    self.right.append(html)
                elif position == 'bottom':
                    self.bottom.append(html)

    def __unicode__(self):
        return self.render('dashboard_panel.html')


class NumberPanel(Component):

    def __init__(self, request, title, number, description, icon=None):
        super(NumberPanel, self).__init__(request)
        self.title = title
        self.number = number
        self.description = description
        self.icon = icon or 'fa-comment-o'

    def __unicode__(self):
        return self.render('number_panel.html')


class NotificationPanel(Component):

    def __init__(self, request, title, count, url, description, icon=None):
        super(NotificationPanel, self).__init__(request)
        self.title = title
        self.count = count
        self.url = url
        self.description = description
        self.icon = icon or 'fa-bell-o'

    def __unicode__(self):
        return self.render('notification_panel.html')


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from djangoplus.admin.models import User
from djangoplus.test import TestCase
from django.conf import settings


class AdminTestCase(TestCase):

    def test_app(self):

        User.objects.create_superuser('admin', None, settings.DEFAULT_PASSWORD)

        self.open('/')
        self.wait(2)

        self.login('admin', settings.DEFAULT_PASSWORD)
        self.click_icon('Configurações')
        self.click_link('Editar Perfil')
        self.enter('Nome', 'Administrador')
        self.enter('Senha', '123')
        self.enter('Confirmação', '123')
        self.click_button('Atualizar Perfil')
        self.logout()
        self.login('admin', '123')
        self.click_icon('Configurações')
        self.click_link('Configurações')
        self.enter('Nome', 'My Project')
        self.click_button('Atualizar')

        self.click_icon('Usuários')
        self.click_link('Cadastrar')
        self.enter('Nome', 'Carlos Breno')
        self.enter('E-mail', 'brenokcc@yahoo.com.br')
        self.enter('Login', 'brenokcc')
        self.enter('Senha', 'senha')
        self.click_button('Cadastrar')
        self.logout()
        self.login('brenokcc', 'senha')
        self.logout()

        self.login('admin', '123')
        self.click_icon('Usuários')
        self.look_at('Carlos Breno')

        self.click_button('Alterar Senha')
        self.enter('Senha', '321')
        self.enter('Confirmação', '321')
        self.click_button('Alterar')
        self.logout()

        self.login('brenokcc', '321')
        self.logout()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.apps import apps
from django.conf import settings
from djangoplus.utils.metadata import get_metadata, get_scope

initialized = False

# user interface
views = []
widgets = []
subset_widgets = []
card_panel_models = []
icon_panel_models = []
subsets = dict()

# roles
role_models = dict()
abstract_role_models = dict()
abstract_role_model_names = dict()

# actions
actions = dict()
class_actions = dict()
fieldset_actions = dict()
subset_actions = dict()

# documentation
workflows = dict()
class_diagrams = dict()
composition_fields = dict()
composition_relations = dict()

# access scope
organization_model = None
unit_model = None
signup_model = None
permissions_by_scope = dict()

# formatters
formatters = dict()

# documentation
last_authenticated_role = None

if not initialized:
    initialized = True
    for model in apps.get_models():
        model_name = model.__name__.lower()
        app_label = get_metadata(model, 'app_label')
        add_shortcut = get_metadata(model, 'add_shortcut')
        list_shortcut = get_metadata(model, 'list_shortcut')
        verbose_name = get_metadata(model, 'verbose_name')
        verbose_name_plural = get_metadata(model, 'verbose_name_plural')
        list_menu = get_metadata(model, 'list_menu')
        role_signup = get_metadata(model, 'role_signup', False)

        field_names = []
        for field in get_metadata(model, 'get_fields'):
            field_names.append(field.name)
            if hasattr(field, 'composition') and field.composition:
                composition_fields[model] = field.name
                if field.rel.to not in composition_relations:
                    composition_relations[field.rel.to] = []
                if model not in composition_relations[field.rel.to]:
                    composition_relations[field.rel.to].append(model)

        if model not in subsets:
            subsets[model] = []
        if model not in subset_actions:
            subset_actions[model] = dict()
        if model not in actions:
            actions[model] = dict()
        if model not in class_actions:
            class_actions[model] = dict()
        if model not in fieldset_actions:
            fieldset_actions[model] = dict()

        if role_signup:
            signup_model = model

        # indexing organization model
        if hasattr(model, 'organization_ptr_id'):
            organization_model = model

        # indexing unit model
        if hasattr(model, 'unit_ptr_id'):
            unit_model = model

        # indexing shortcuts
        if add_shortcut:
            icon_panel_models.append((model, add_shortcut))
        if list_shortcut:
            card_panel_models.append((model, list_shortcut))

        # indexing the views generated from model classes
        url = '/list/{}/{}/'.format(app_label, model_name)
        icon = None
        if list_menu and model not in composition_fields:
            menu_groups = ()
            if type(list_menu) == tuple:
                if len(list_menu) == 2:
                    menu, icon = list_menu
                else:
                    menu, icon, menu_groups = list_menu
            else:
                menu, icon = list_menu, get_metadata(model, 'icon')
            permission = '{}.list_{}'.format(app_label, model_name)
            # if issubclass(model, Model):
            item = dict(url=url, can_view=permission, menu=menu, icon=icon, add_shortcut=False, groups=menu_groups)
            views.append(item)

        # indexing the subsets defined in the manager classes

        for attr_name in dir(model.objects.get_queryset()):
            attr = getattr(model.objects.get_queryset(), attr_name)
            if hasattr(attr, '_metadata'):
                metadata_type = attr._metadata['{}:type'.format(attr_name)]
                if metadata_type == 'subset':
                    subset_title = attr._metadata['{}:title'.format(attr_name)]
                    subset_name = attr._metadata['{}:name'.format(attr_name)]
                    subset_alert = attr._metadata['{}:alert'.format(attr_name)]
                    subset_notify = attr._metadata['{}:notify'.format(attr_name)]
                    subset_can_view = attr._metadata['{}:can_view'.format(attr_name)]
                    subset_order = attr._metadata['{}:order'.format(attr_name)]
                    subset_menu = attr._metadata['{}:menu'.format(attr_name)]
                    subset_workflow = attr._metadata['{}:sequence'.format(attr_name)]
                    subset_url = '{}{}/'.format(url, attr.im_func.func_name)

                    item = dict(title=subset_title, name=attr_name, function=attr, url=subset_url, can_view=subset_can_view, menu=subset_menu, icon=icon, alert=subset_alert, notify=subset_notify, actions=subset_actions, order=subset_order)
                    subsets[model].append(item)

                    if subset_workflow:
                        role = subset_can_view and subset_can_view[0] or 'Superusuário'
                        if attr_name == 'all':
                            activity_description = 'Listar {}'.format(verbose_name_plural,)
                        else:
                            activity_description = 'Listar {}: {}'.format(verbose_name_plural, subset_title)
                        workflows[subset_workflow] = dict(activity=activity_description, role=role, model=None)

                else:
                    widget_title = attr._metadata['{}:verbose_name'.format(attr_name)]
                    widget_can_view = attr._metadata['{}:can_view'.format(attr_name)]
                    widget_position = attr._metadata['{}:position'.format(attr_name)]
                    widget_formatter = attr._metadata['{}:formatter'.format(attr_name)]
                    widget = dict(title=widget_title, model=model, function=attr_name, can_view=widget_can_view, position=widget_position, formatter=widget_formatter)
                    subset_widgets.append(widget)

        # indexing the actions refered in fieldsets
        if hasattr(model, 'fieldsets'):
            for title, info in model.fieldsets:
                if title not in fieldset_actions[model]:
                    fieldset_actions[model][title] = []
                for action_name in info.get('actions', []):
                    fieldset_actions[model][title].append(action_name)

        # indexing the actions defined in models
        for attr in dir(model):
            if attr[0] != '_' and attr not in field_names:
                function = getattr(model, attr)
                if hasattr(function, '_action'):
                    action = getattr(function, '_action')
                    action_group = action['group']

                    action_can_execute = action['can_execute']
                    action_title = action['title']
                    action_workflow = action['sequence']
                    action_menu = action['menu']
                    action_inline = action['inline']
                    view_name = action['view_name']
                    if action_group not in actions[model]:
                        actions[model][action_group] = dict()
                    actions[model][action_group][view_name] = action
                    if action_inline:
                        action_subset = action_inline is not True and action_inline or None
                        if action_subset not in subset_actions[model]:
                            subset_actions[model][action_subset] = []
                        subset_actions[model][action_subset].append(attr)
                    if action_workflow:
                        role = action_can_execute and action_can_execute[0] or 'Superusuário'
                        workflows[action_workflow] = dict(activity=action_title, role=role, model=verbose_name)
                    if action_menu:
                        url = '/action/{}/{}/{}/'.format(get_metadata(model, 'app_label'), model.__name__.lower(), attr)
                        action_view = dict(title=action_title, function=None, url=url, can_view=action_can_execute, menu=action_menu, icon=None,
                              style='ajax', add_shortcut=False, doc=function.__doc__, sequence=None)
                        views.append(action_view)

        # indexing the actions defined in managers
        for attr_name in dir(model.objects.get_queryset()):
            if not attr_name[0] == '_':
                attr = getattr(model.objects.get_queryset(), attr_name)
                if hasattr(attr, '_action'):
                    action = getattr(attr, '_action')
                    action_title = action['title']
                    action_can_execute = action['can_execute']
                    action_group = action['group']
                    view_name = action['view_name']
                    action_subset = action['inline']
                    action_workflow = action['sequence']
                    if action_group not in class_actions[model]:
                        class_actions[model][action_group] = dict()
                    class_actions[model][action_group][view_name] = action

                    if action_subset not in subset_actions[model]:
                        subset_actions[model][action_subset] = []
                    subset_actions[model][action_subset].append(view_name)

                    if action_workflow:
                        role = action_can_execute and action_can_execute[0] or 'Superusuário'
                        workflows[action_workflow] = dict(activity=action_title, role=role, model=verbose_name)

    # indexing the actions, views and widgets in views module
    for app_label in settings.INSTALLED_APPS:
            try:
                module = __import__('{}.formatters'.format(app_label), fromlist=list(map(str, app_label.split('.'))))
                for attr_name in dir(module):
                    module_attr = getattr(module, attr_name)
                    if callable(module_attr):
                        formatters[attr_name] = module_attr
            except ImportError:
                pass

            try:
                module = __import__('{}.views'.format(app_label), fromlist=list(map(str, app_label.split('.'))))
                for attr_name in dir(module):
                    function = getattr(module, attr_name)
                    # indexing the actions
                    if hasattr(function, '_action'):
                        action = function._action
                        action_group = action['group']
                        action_model = action['model']
                        action_function = action['function']
                        action_name = action['view_name']
                        action_title = action['title']
                        action_workflow = action['sequence']
                        action_can_execute = action['can_execute']
                        action_inline = action['inline']
                        action_menu = action['menu']

                        if action_inline:
                            action_subset = action_inline is not True and action_inline or None
                            if action_subset not in subset_actions[action_model]:
                                subset_actions[action_model][action_subset] = []
                            subset_actions[action_model][action_subset].append(action_name)

                        if action_workflow:
                            role = action_can_execute and action_can_execute[0] or 'Superusuário'
                            action_model_verbose_name = get_metadata(action_model, 'verbose_name')
                            workflows[action_workflow] = dict(activity=action_title, role=role, model=action_model_verbose_name)
                        if action_function.func_code.co_argcount > 1:
                            if action_group not in actions[action_model]:
                                actions[action_model][action_group] = dict()
                            actions[action_model][action_group][action_name] = action
                        else:
                            if action_model not in class_actions:
                                class_actions[action_model] = dict()
                            if action_group not in class_actions[action_model]:
                                class_actions[action_model][action_group] = dict()
                            class_actions[action_model][action_group][action_name] = action
                    # indexing the views
                    elif hasattr(function, '_view'):
                        views.append(function._view)
                        view_title = function._view['title']
                        view_workflow = function._view['sequence']
                        view_can_view = function._view['can_view']
                        if view_workflow:
                            role = view_can_view and view_can_view[0] or 'Superusuário'
                            workflows[view_workflow] = dict(activity=view_title, role=role, model=None)
                    # indexing the widgets
                    elif hasattr(function, '_widget'):
                        widgets.append(function._widget)
            except ImportError:
                pass

    for model in apps.get_models():
        app_label = get_metadata(model, 'app_label')
        verbose_name = get_metadata(model, 'verbose_name')
        role_username = get_metadata(model, 'role_username')
        add_label = get_metadata(model, 'add_label', None)
        workflow = get_metadata(model, 'sequence', 0)
        diagram_classes = get_metadata(model, 'class_diagram', None)

        # indexing role models
        if role_username:
            role_models[model] = dict(username_field=role_username, scope=get_scope(model, organization_model, unit_model), name=verbose_name)
        for subclass in model.__subclasses__():
            subclass_role_username = get_metadata(subclass, 'role_username')
            if subclass_role_username:
                subclass_verbose_name = get_metadata(subclass, 'verbose_name')
                role_models[subclass] = dict(username_field=subclass_role_username, scope=get_scope(subclass, organization_model, unit_model), name=subclass_verbose_name)
                if model not in abstract_role_models:
                    abstract_role_models[model] = []
                    abstract_role_model_names[verbose_name] = []
                abstract_role_models[model].append(subclass)
                abstract_role_model_names[verbose_name].append(subclass_verbose_name)

        permission_by_scope = dict()
        for scope in ('role', 'unit', 'organization'):
            for permission_name in ('edit', 'add', 'delete', 'list'):
                permission_key = '{}_by_{}'.format(permission_name, scope)
                for group_name in get_metadata(model, 'can_{}'.format(permission_key), (), iterable=True):
                    if permission_key not in permission_by_scope:
                        permission_by_scope[permission_key] = []
                    if group_name in abstract_role_model_names:
                        for concrete_group_name in abstract_role_model_names[group_name]:
                            permission_by_scope[permission_key].append(concrete_group_name)
                    else:
                        permission_by_scope[permission_key].append(group_name)
            for group_name in get_metadata(model, 'can_admin_by_{}'.format(scope), (), iterable=True):
                for permission_name in ('edit', 'add', 'delete', 'list'):
                    permission_key = '{}_by_{}'.format(permission_name, scope)
                    if permission_key not in permission_by_scope:
                        permission_by_scope[permission_key] = []
                    if group_name not in permission_by_scope[permission_key]:
                        if group_name in abstract_role_model_names:
                            for concrete_group_name in abstract_role_model_names[group_name]:
                                permission_by_scope[permission_key].append(concrete_group_name)
                        else:
                            permission_by_scope[permission_key].append(group_name)

        for permission_name in ('edit', 'add', 'delete', 'list'):
            permission_key = permission_name
            for group_name in get_metadata(model, 'can_{}'.format(permission_name), (), iterable=True):
                if permission_key not in permission_by_scope:
                    permission_by_scope[permission_key] = []
                if group_name not in permission_by_scope[permission_key]:
                    permission_by_scope[permission_key].append(group_name)
        for group_name in get_metadata(model, 'can_admin', (), iterable=True):
            for permission_name in ('edit', 'add', 'delete', 'list'):
                permission_key = permission_name
                if permission_key not in permission_by_scope:
                    permission_by_scope[permission_key] = []
                if group_name not in permission_by_scope[permission_key]:
                    permission_by_scope[permission_key].append(group_name)

        for actions_dict in (actions, class_actions):
            for category in actions_dict[model]:
                for key in actions_dict[model][category].keys():
                    name = actions_dict[model][category][key]['title']
                    view_name = actions_dict[model][category][key]['view_name']
                    can_execute = []
                    for scope in ('', 'role', 'unit', 'organization'):
                        scope = scope and '_by_{}'.format(scope) or scope
                        for group_name in actions_dict[model][category][key].get('can_execute{}'.format(scope)) or ():
                            permission_key = '{}{}'.format(view_name, scope)
                            if permission_key not in permission_by_scope:
                                permission_by_scope[permission_key] = []
                            permission_by_scope[permission_key].append(group_name)

        if permission_by_scope:
            permissions_by_scope[model] = permission_by_scope

        if workflow:
            role = permission_by_scope.get('add') and permission_by_scope.get('add')[0] or None
            if not role:
                role = permission_by_scope.get('add_by_role') and permission_by_scope.get('add_by_role')[0] or None
            if not role:
                role = permission_by_scope.get('add_by_unit') and permission_by_scope.get('add_by_unit')[0] or None
            if not role:
                role = permission_by_scope.get('add_by_organization') and permission_by_scope.get('add_by_organization')[0] or None
            if not role:
                role = 'Superusuário'

            if model in composition_fields:
                related_model = getattr(model, composition_fields[model]).field.rel.to
                related_verbose_name = get_metadata(related_model, 'verbose_name')
                related_add_label = get_metadata(model, 'add_label')
                if related_add_label:
                    activity = '{} em {}'.format(related_add_label, related_verbose_name)
                else:
                    activity = 'Adicionar {} em {}'.format(verbose_name, related_verbose_name)
                workflows[workflow] = dict(activity=activity, role=role, model=None)
            else:
                if add_label:
                    activity = add_label
                else:
                    activity = '{} {}'.format('Cadastrar', verbose_name)
                workflows[workflow] = dict(activity=activity, role=role, model=None)

        if diagram_classes is not None:
            class_diagrams[verbose_name] = [model]
            if type(diagram_classes) == bool and diagram_classes:
                for field in model._meta.get_fields():
                    if hasattr(field, 'rel') and hasattr(field.rel, 'to') and field.rel.to:
                        if field.rel.to not in class_diagrams[verbose_name]:
                            class_diagrams[verbose_name].append(field.rel.to)
            else:
                for model_name in diagram_classes:
                    try:
                        extra_model = apps.get_model(app_label, model_name)
                    except LookupError:
                        for extra_model in apps.get_models():
                            if extra_model.__name__.lower() == model_name:
                                break
                    if extra_model not in class_diagrams[verbose_name]:
                        class_diagrams[verbose_name].append(extra_model)

    keys = workflows.keys()
    keys.sort()
    l = []
    for key in keys:
        l.append(workflows[key])
    workflows = l



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re
import datetime
import unicodedata
from decimal import Decimal
from django.utils.safestring import mark_safe
from django.db.models.fields.files import FieldFile, ImageFieldFile as DjangoImageFieldFile


def normalyze(nome):
    nome = unicode(nome)

    if nome.isupper():
        return nome

    ponto = '\.'
    ponto_espaco = '. '
    espaco = ' '
    regex_multiplos_espacos = '\s+'
    regex_numero_romano = '^M{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})$'

    nome = re.sub(ponto, ponto_espaco, nome)  # colocando espaço após nomes abreviados
    nome = re.sub(regex_multiplos_espacos, espaco, nome)  # retirando espaços múltiplos
    nome = nome.title()  # alterando os nomes para CamelCase
    partes_nome = nome.split(espaco)  # separando as palavras numa lista
    excecoes = [
        'de', 'di', 'do', 'da', 'dos', 'das', 'dello', 'della', 'dalla',
        'dal', 'del', 'e', 'em', 'na', 'no', 'nas', 'nos', 'van', 'von', 'y', 'para', 'pela', 'pelo', 'por',
    ]

    resultado = []

    for palavra in partes_nome:
        if palavra.lower() in excecoes:
            resultado.append(palavra.lower())
        elif re.match(regex_numero_romano, palavra.upper()):
            resultado.append(palavra.upper())
        else:
            resultado.append(palavra)

    nome = espaco.join(resultado)
    return nome


def format_bool(value):
    return value and '<span class="label label-success">Sim</span>' or '<span class="label label-danger">Não</span>'


def format_value(value, html=True):
    from djangoplus.db.models.fields import ImageFieldFile
    if value is None or value == '' or value == ():
        return '-'
    elif type(value) == str:
        return value
    elif isinstance(value, Decimal):
        if hasattr(value, 'decimal3'):
            return format_decimal3(value)
        return format_decimal(value)
    elif isinstance(value, ImageFieldFile) or isinstance(value, DjangoImageFieldFile):
        value = unicode(value)
        url = '/static/' in value and value or '/media/{}'.format(value)
        return html and mark_safe('<img width="50px" class="materialboxed" src="{}"/>'.format(url)) or value
    elif isinstance(value, FieldFile):
        value = unicode(value)
        url = '/static/' in value and value or '/media/{}'.format(value)
        file_name = value.split('/')[-1]
        if url.lower().endswith('.pdf'):
            return html and mark_safe(
                '<a class="ajax pdf" href="{}">{}</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href="{}"><i class="mdi-file-file-download"></i></a>'.format(
                url, file_name, url)) or file_name
        else:
            return html and mark_safe('<a target="_blank" href="{}">{}</a>'.format(url, file_name)) or url
    elif isinstance(value, bool):
        return value and 'Sim' or 'Não'
    elif value.__class__ == datetime.date:
        return value.strftime('%d/%m/%Y')
    elif value.__class__ == datetime.datetime:
        return value.strftime('%d/%m/%Y %H:%M')
    elif type(value).__name__ in ('QuerySet',) or type(value) == list:
        if html:
            l = ['<ul>']
            for obj in value:
                l.append('<li>{}</li>'.format(obj))
            l.append('</ul>')
            return mark_safe(''.join(l))
        else:
            l = []
            for obj in value:
                l.append(unicode(obj))
            return ', '.join(l)
    elif isinstance(value, tuple):
        return '{} {}'.format(value[0], value[1])
    else:
        return unicode(value)


def split_thousands(value, sep='.'):
    if not isinstance(value, basestring):
        value = str(value)
    negativo = False
    if '-' in value:
        value = value.replace('-', '')
        negativo = True
    if len(value) <= 3:
        if negativo:
            return '- ' + value
        else:
            return value
    if negativo:
        return '- ' + split_thousands(value[:-3], sep) + sep + value[-3:]
    else:
        return split_thousands(value[:-3], sep) + sep + value[-3:]


def format_decimal(value):
    value = str(value)
    if '.' in value:
        reais, centavos = value.split('.')
        if len(centavos) == 1:
            centavos = '{}0'.format(centavos)
        elif len(centavos) > 2:
            centavos = centavos[0:2]
    else:
        reais = value
        centavos = '00'
    reais = split_thousands(reais)
    return reais + ',' + centavos


def format_decimal3(value):
    value = str(value)
    if '.' in value:
        reais, centavos = value.split('.')
        centavos = '{}000'.format(centavos)
        if len(centavos) == 1:
            centavos = '{}0'.format(centavos)
        elif len(centavos) > 3:
            centavos = centavos[0:3]
    else:
        reais = value
        centavos = '000'
    reais = split_thousands(reais)
    return reais + ',' + centavos


def to_ascii(txt, codif='utf-8'):
    if not isinstance(txt, basestring):
        txt = unicode(txt)
    if isinstance(txt, unicode):
        txt = txt.encode('utf-8')
    return unicodedata.normalize('NFKD', txt.decode(codif)).encode('ASCII', 'ignore')


import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='djangoplus',
    version='0.0.40',
    packages=find_packages(),
    install_requires=['Django==1.11', 'pycrypto==2.6.1', 'python-dateutil==2.5.3', 'gunicorn==19.5.0', 'selenium==2.53.2', 'xlwt==1.0.0', 'xlrd==0.9.4', 'unicodecsv==0.14.1', 'dropbox==6.5.0', 'html5lib==1.0b8', 'Pillow==3.2.0', 'xhtml2pdf==0.0.6', 'Fabric==1.14.0'],
    scripts=['djangoplus/bin/startproject', 'djangoplus/bin/runserver', 'djangoplus/bin/sync'],
    include_package_data=True,
    license='BSD License',
    description='Metadata-based web framework for the development of management information systems',
    long_description=README,
    url='http://djangoplus.net/',
    author='Breno Silva',
    author_email='brenokcc@yahoo.com.br',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.utils import termcolors


def bold(text):
    return termcolors.make_style(fg='black', opts=('bold',))(text)


def info(text):
    return termcolors.make_style(fg='cyan')(text)


def error(text):
    return termcolors.make_style(fg='red', opts=('bold',))(text)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
from django.forms import widgets
from django.utils.safestring import mark_safe
from djangoplus.utils.metadata import get_metadata
from django.template.loader import render_to_string
from djangoplus.utils.serialization import dumps_qs_query


INIT_SCRIPT = '''{html}
<script>
    $("#id_{name}").select2({{
        allowClear: true,
        language: 'pt-BR',
        {templates}
        escapeMarkup: function (markup) {{ var links = markup=='Nenhum resultado encontrado'? '<div>{links}</dev>' : ''; return markup + links; }}}}).on('select2:unselecting',
        function() {{
            $(this).data('unselecting', true);}}).on(
            'select2:opening',
            function(e) {{
                if ($(this).data('unselecting')) {{
                    $(this).removeData('unselecting');
                    e.preventDefault();
                }}
            }}
        );
    function load{function_name}(value){{
        $('#id_{name}').val(value);
        $('#id_{name}').trigger("change");
    }}
</script>'''

AJAX_INIT_SCRIPT = '''{html}
<script>
    window['qs_{name}'] = {{ 'qs' : '{qs_dump}' }}
    $('#id_{name}').select2({{
        allowClear: true,
        language: 'pt-BR',
        escapeMarkup: function (markup) {{ var links = markup=='Nenhum resultado encontrado'? '<div>{links}</dev>' : ''; return markup + links; }},
        ajax: {{
            delay: 500,
            dataType: 'json',
            type: 'POST',
            url:'/autocomplete/{app_label}/{model_name}/',
            data: function (params) {{return {{qs: window['qs_{name}'], q:params.term}}}},
            cache: true,
            success: function(data){{
                var values = $('#id_{name}').val();
                //$('#id_{name}').empty();
                $('#id_{name}').append($('<option></option>').attr("value", "").text(""));
                for(var i=0; i<data.results.length; i++){{
                    var option = $('<option></option>').attr("value", data.results[i].id).text(data.results[i].text)
                    //if(values.indexOf(String(data.results[i].id))>-1) option.attr("selected", true)
                    $('#id_{name}').append(option);
                }}
            }}
        }},
        minimumInputLength: 1,
        templateResult: function (item) {{if (item.loading) return item.text; return item.html;}}
        }}
    );
    function load{function_name}(value, text){{
        var option = $('<option></option>').attr("value", value).text(text).attr("selected", true);
        $('#id_{name}').append(option);
    }}
</script>
'''

RELOAD_SCRIPT = '''
<script>
    function reload_({function_name}(){{
        var pk = $('#id_{field_name}').val()
        if(!pk) pk = 0;
        $.ajax({{url:"/reload_options/{app_label}/{model_name}/{value}/{lookup}/"+pk+"/{lazy}/", dataType:'json', success:function( data ) {{
            if({lazy}){{
                window['qs_{name}']['qs'] = data.qs;
            }} else {{
                $('#id_{name}').select2('destroy').empty().select2({{allowClear: true, data: data.results}});
                $('#id_{name}').val('{value}'.split('_'));
            }}
            $('#id_{name}').trigger("change");
        }}}});
    }}
    $('#id_{field_name}').on('change', function(e) {{
        reload_{function_name}();
    }});
    reload_{function_name}();
</script>
'''

ADD_LINK = '<a class="pull-right popup" style="padding:5px" href="javascript:" onclick="$(\\\'#id_{}\\\').select2(\\\'close\\\');popup(\\\'/add/{}/{}/?select=id_{}\\\');"><i class="fa fa-plus">\</i>Adicionar {}</a>'


class SelectWidget(widgets.Select):

    class Media:
        css = {'all': ('/static/css/select2.min.css',)}
        js = ('/static/js/select2.min.js', '/static/js/i18n/pt-BR.js')

    def __init__(self, *args, **kwargs):
        super(SelectWidget, self).__init__(*args, **kwargs)
        self.lazy = False
        self.form_filters = []

    def render(self, name, value, attrs=None):

        if 'HEADLESS' in os.environ:
            self.lazy = False

        attrs['class'] = 'form-control'
        if 'data-placeholder' not in self.attrs:
            attrs['data-placeholder'] = ' '
        queryset = None
        if self.lazy and hasattr(self.choices, 'queryset'):
            queryset = self.choices.queryset
            self.choices.queryset = self.choices.queryset.model.objects.filter(pk=value or 0)

        model = None
        links = []
        templates = []
        if hasattr(self.choices, 'queryset'):
            model = self.choices.queryset.model
            models = model.__subclasses__() or [model]

            if not self.lazy:
                select_template = get_metadata(model, 'select_template')
                select_display = get_metadata(model, 'select_display')
                if select_template or select_display:
                    templates_var_name = name.replace('-', '_')
                    templates.append('templateResult: function (item) {{{}_templates = Array();'.format(templates_var_name))
                    if hasattr(self.choices.queryset.model, 'get_tree_index_field'):
                        tree_index_field = self.choices.queryset.model.get_tree_index_field()
                        if tree_index_field:
                            self.choices.queryset = self.choices.queryset.order_by(tree_index_field.name)
                    for obj in self.choices.queryset:
                        obj_html = render_to_string(select_template or 'select_template.html', dict(obj=obj, select_display=select_display)) or unicode(obj)
                        templates.append('{}_templates[{}] = \'{}\';'.format(templates_var_name, obj.pk, obj_html.replace('\n', '')))
                    templates.append('return {}_templates[item.id];}},'.format(templates_var_name))

            if hasattr(self, 'user'):
                for tmp in models:
                    class_name = tmp.__name__.lower()
                    app_label = get_metadata(tmp, 'app_label')
                    perm = '{}.add_{}'.format(app_label, class_name)
                    if self.user.has_perm(perm):
                        links.append(ADD_LINK.format(name, app_label, class_name, name, get_metadata(tmp, 'verbose_name')))

        html = super(SelectWidget, self).render(name, value, attrs)
        html = html.replace('---------', '')
        function_name = name.replace('-', '__')
        if model and self.lazy:
            app_label = get_metadata(model, 'app_label')
            model_name = model.__name__.lower()
            qs_dump = dumps_qs_query(queryset)
            html = AJAX_INIT_SCRIPT.format(html=html, name=name, function_name=function_name, qs_dump=qs_dump, app_label=app_label, model_name=model_name, links=''.join(links))
        else:
            html = INIT_SCRIPT.format(html=html, name=name, function_name=function_name, links=''.join(links), templates=''.join(templates))

        if model:
            value = value or 0
            lazy = self.lazy and 1 or 0
            for field_name, lookup in self.form_filters:
                if '-' in name:
                    field_name = '{}-{}'.format(name.split('-')[0], field_name)
                app_label = get_metadata(model, 'app_label')
                model_name = model.__name__.lower()
                function_name = name.replace('-', '__')
                reload_script = RELOAD_SCRIPT.format(
                    function_name=function_name, field_name=field_name, app_label=app_label, model_name=model_name,
                    value=value, lookup=lookup, lazy=lazy, name=name
                )
                html = '{} {}'.format(html, reload_script)
        return mark_safe(html)


class SelectMultipleWidget(widgets.SelectMultiple):

    class Media:
        css = {'all': ('/static/css/select2.min.css',)}
        js = ('/static/js/select2.min.js', '/static/js/i18n/pt-BR.js')

    def __init__(self, *args, **kwargs):
        super(SelectMultipleWidget, self).__init__(*args, **kwargs)
        self.lazy = False
        self.form_filters = []

    def render(self, name, value, attrs=None):

        if 'HEADLESS' in os.environ:
            self.lazy = False

        attrs['class'] = 'form-control'
        attrs['data-placeholder'] = ' '
        queryset = None
        templates = []
        if hasattr(self.choices, 'queryset'):
            queryset = self.choices.queryset.all()
            if self.lazy:
                self.choices.queryset = self.choices.queryset.model.objects.filter(pk__in=value or [])
            else:
                select_template = get_metadata(queryset.model, 'select_template')
                select_display = get_metadata(queryset.model, 'select_display')
                if select_template or select_display:
                    templates_var_name = name.replace('-', '_')
                    templates.append('templateResult: function (item) {{{}_templates = Array();'.format(templates_var_name))
                    if hasattr(self.choices.queryset.model, 'get_tree_index_field'):
                        tree_index_field = self.choices.queryset.model.get_tree_index_field()
                        if tree_index_field:
                            self.choices.queryset = self.choices.queryset.order_by(tree_index_field.name)
                    for obj in self.choices.queryset.all():
                        obj_html = render_to_string(select_template or 'select_template.html', dict(obj=obj, select_display=select_display)) or unicode(obj)
                        templates.append('{}_templates[{}] = \'{}\';'.format(templates_var_name, obj.pk, obj_html.replace('\n', '')))
                    templates.append('return {}_templates[item.id];}},'.format(templates_var_name))
        html = super(SelectMultipleWidget, self).render(name, value, attrs)
        links = []
        if queryset:
            models = queryset.model.__subclasses__() or [queryset.model]
            if hasattr(self, 'user'):
                for tmp in models:
                    class_name = tmp.__name__.lower()
                    app_label = get_metadata(tmp, 'app_label')
                    perm = '{}.add_{}'.format(app_label, class_name)
                    if self.user.has_perm(perm):
                        links.append(ADD_LINK.format(name, app_label, class_name, name, get_metadata(tmp, 'verbose_name')))

        function_name = name.replace('-', '__')
        if queryset.model and self.lazy:
            app_label = get_metadata(queryset.model, 'app_label')
            model_name = queryset.model.__name__.lower()
            qs_dump = dumps_qs_query(queryset)
            html = AJAX_INIT_SCRIPT.format(html=html, name=name, function_name=function_name, qs_dump=qs_dump, app_label=app_label, model_name=model_name, links=''.join(links))
        else:
            html = INIT_SCRIPT.format(html=html, name=name, function_name=function_name, links=''.join(links), templates=''.join(templates))

        if queryset.model:
            l = []
            if value:
                for pk in value:
                    l.append(str(pk))
            else:
                l.append('0')
            lazy = self.lazy and 1 or 0
            for field_name, lookup in self.form_filters:
                app_label = get_metadata(queryset.model, 'app_label')
                model_name = queryset.model.__name__.lower()
                value = '_'.join(l)
                function_name = name.replace('-', '__')
                reload_script = RELOAD_SCRIPT.format(function_name=function_name, field_name=field_name,
                    app_label=app_label, model_name=model_name, value=value, lookup=lookup, lazy=lazy, name=name)
                html = '{} {}'.format(html, reload_script)

        return mark_safe(html)
    

class NullBooleanSelect(widgets.NullBooleanSelect):

    class Media:
        css = {'all': ('/static/css/select2.min.css',)}
        js = ('/static/js/select2.min.js', '/static/js/i18n/pt-BR.js')

    def render(self, name, value, attrs=None):
        attrs['class'] = 'form-control'
        if 'data-placeholder' not in self.attrs:
            attrs['data-placeholder'] = ' '
        function_name = name.replace('-', '__')
        html = super(NullBooleanSelect, self).render(name, value, attrs)
        html = INIT_SCRIPT.format(html=html, name=name, function_name=function_name, templates='', links='')
        return mark_safe(html)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# Generated by Django 1.10 on 2016-10-01 17:50
from __future__ import unicode_literals

from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0003_auto_20160914_1821'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='settings',
            name='default_color',
        ),
        migrations.AddField(
            model_name='settings',
            name='sidebar_fixed',
            field=djangoplus.db.models.fields.BooleanField(default=False, verbose_name='Fixo'),
        ),
        migrations.AddField(
            model_name='settings',
            name='sidebar_hidden',
            field=djangoplus.db.models.fields.BooleanField(default=False, verbose_name='Escondido'),
        ),
        migrations.AddField(
            model_name='settings',
            name='sidebar_mini',
            field=djangoplus.db.models.fields.BooleanField(default=False, verbose_name='Compacto'),
        ),
        migrations.AddField(
            model_name='settings',
            name='theme',
            field=djangoplus.db.models.fields.CharField(choices=[[b'skin-6', b'skin-6'], [b'skin-5', b'skin-5']], default=b'skin-6', max_length=255, verbose_name='Tema'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf import settings
from djangoplus.ui import Component
from djangoplus.utils import permissions
from django.utils.safestring import mark_safe


class Menu(Component):
    def __init__(self, request, app_settings=None):
        super(Menu, self).__init__(request)
        self.subitems = dict()
        self.settings = app_settings

    def add(self, description, url, icon=None, style='ajax'):
        url = '/breadcrumbs/reset{}'.format(url)
        levels = description.split('::')
        for i, level in enumerate(levels):
            levels[i] = level.strip()

        if levels[0] not in self.subitems:
            self.subitems[levels[0]] = dict(urls=[], subitems=dict(), icon=None)
        if not self.subitems[levels[0]]['icon']:
            self.subitems[levels[0]]['icon'] = icon

        if len(levels) == 1:
            self.subitems[levels[0]]['urls'].append((url, style))
        else:
            if levels[1] not in self.subitems[levels[0]]['subitems']:
                self.subitems[levels[0]]['subitems'][levels[1]] = dict(urls=[], subitems=dict())
            if len(levels) == 2:
                self.subitems[levels[0]]['subitems'][levels[1]]['urls'].append((url, style))
            else:
                if levels[2] not in self.subitems[levels[0]]['subitems'][levels[1]]['subitems']:
                    self.subitems[levels[0]]['subitems'][levels[1]]['subitems'][levels[2]] = dict(urls=[], subitems=dict())
                if len(levels) == 3:
                    self.subitems[levels[0]]['subitems'][levels[1]]['subitems'][levels[2]]['urls'].append((url, style))
                else:
                    if levels[3] not in self.subitems[levels[0]]['subitems'][levels[1]]['subitems'][levels[2]]['subitems']:
                        self.subitems[levels[0]]['subitems'][levels[1]]['subitems'][levels[2]]['subitems'][levels[3]] = dict(urls=[])
                    self.subitems[levels[0]]['subitems'][levels[1]]['subitems'][levels[2]]['subitems'][levels[3]]['urls'].append((url, style))

    def load(self):
        if settings.DEBUG or 'side_menu' not in self.request.session:
            from djangoplus.cache import loader
            for item in loader.views:
                if item['menu']:
                    can_view = permissions.check_group_or_permission(self.request, item['can_view'])
                    if can_view and 'groups' in item:
                        can_view = permissions.check_group_or_permission(self.request, item['groups'])
                    if can_view:
                        self.add(item['menu'], item['url'], item['icon'], item.get('style', 'ajax'))

            for cls, itens in loader.subsets.items():
                for item in itens:
                    if permissions.check_group_or_permission(self.request, item['can_view']):
                        if False:  # TODO False
                            self.add(item['menu'], item['url'], item['icon'], 'ajax')

            self.request.session['side_menu'] = self.render('menu.html')
            self.request.session['side_menu_size'] = len(self.subitems.keys())
            self.request.session.save()

    def __unicode__(self):
        return mark_safe(self.request.session['side_menu'])


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import hashlib
import binascii
from Crypto import Random
from django.conf import settings
from Crypto.Cipher import AES


class AESCipher(object):
    def __init__(self, key):
        self.bs = 32
        self.key = hashlib.sha256(key.encode()).digest()

    def encrypt(self, raw):
        raw = self._pad(raw)
        iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return binascii.hexlify(iv + cipher.encrypt(raw))

    def decrypt(self, enc):
        enc = binascii.unhexlify(enc)
        iv = enc[:AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return self._unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')

    def _pad(self, s):
        return s + (self.bs - len(s) % self.bs) * chr(self.bs - len(s) % self.bs)

    @staticmethod
    def _unpad(s):
        return s[:-ord(s[len(s) - 1:])]

AESCIPHER = AESCipher(settings.SECRET_KEY)


def encrypt(text):
    return AESCIPHER.encrypt(text)


def decrypt(text):
    return AESCIPHER.decrypt(text)
